package benchmark.graphcoloring.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import benchmark.graphcoloring.URGCGenerator;
import gui.generators.Generator;
import util.Commands;

public class GCGeneratorPanel extends Generator {

	public static final long serialVersionUID = 1L;
	private JPanel principalPanel = null;
	public JLabel nbInstanceLabel = null;
	public JLabel nbAgentLabel = null;
	public JLabel domainSizeLabel = null;
	public JLabel densityLabel = null;
	public JLabel fromLabel = null;
	public JLabel toLabel = null;
	public JLabel stepLabel = null;
	public JSpinner minDensity = null;
	public JSpinner maxDensity = null;
	public JSpinner stepDensity = null;
	private JLabel nbAgentsLabel = null;
	private JSpinner minAgent;
	private JSpinner maxAgent = null;
	private JSpinner stepAgent = null;
	private JSpinner minNbColors = null;
	private JSpinner nbInstance = null;
	private JComboBox format = null;
	private JLabel nbVarsPerAgentLabel = null;
	private JSpinner nbVarsPerAgent = null;
	private JSpinner maxNbColors = null;
	private JSpinner stepNbColors = null;

	/**
	 * This is the default constructor
	 */
	public GCGeneratorPanel() {
		super();
		this.add(this.getPrincipalPanel(), BorderLayout.CENTER);
	}

	/**
	 * This method initializes principalPanel
	 * 
	 * @return simulation.gui.RandomTKCPanel
	 */
	protected JPanel getPrincipalPanel() {
		if (principalPanel == null) {
			principalPanel = new JPanel(null);
			principalPanel.setPreferredSize(new Dimension(400, 375));
			principalPanel.setMinimumSize(new Dimension(400, 375));
			domainSizeLabel = new JLabel();
			domainSizeLabel.setBounds(new Rectangle(25, 140, 120, 22));
			domainSizeLabel.setText("Number of colors");
			domainSizeLabel.setHorizontalAlignment(JLabel.RIGHT);
			densityLabel = new JLabel();
			densityLabel.setBounds(new Rectangle(45, 180, 100, 22));
			densityLabel.setText("Graph density");
			densityLabel.setHorizontalAlignment(JLabel.RIGHT);
			nbAgentsLabel = new JLabel();
			nbAgentsLabel.setBounds(new Rectangle(25, 220, 120, 22));
			nbAgentsLabel.setText("Number of agents");
			nbAgentsLabel.setHorizontalAlignment(JLabel.RIGHT);
			fromLabel = new JLabel();
			fromLabel.setBounds(new Rectangle(150, 105, 55, 22));
			fromLabel.setText("From");
			fromLabel.setHorizontalAlignment(JLabel.CENTER);
			toLabel = new JLabel();
			toLabel.setBounds(new Rectangle(220, 105, 55, 22));
			toLabel.setText("To");
			toLabel.setHorizontalAlignment(JLabel.CENTER);
			stepLabel = new JLabel();
			stepLabel.setBounds(new Rectangle(290, 105, 55, 22));
			stepLabel.setText("Step");
			stepLabel.setHorizontalAlignment(JLabel.CENTER);
			nbInstanceLabel = new JLabel();
			nbInstanceLabel.setBounds(new Rectangle(90, 60, 150, 22));
			nbInstanceLabel.setText("Number of instances");
			nbInstanceLabel.setHorizontalAlignment(JLabel.RIGHT);
			principalPanel.add(nbInstanceLabel);
			principalPanel.add(domainSizeLabel);
			principalPanel.add(densityLabel);
			principalPanel.add(fromLabel);
			principalPanel.add(toLabel);
			principalPanel.add(stepLabel);
			principalPanel.add(nbAgentsLabel);
			principalPanel.add(getMinDensity());
			principalPanel.add(getMaxDensity());
			principalPanel.add(getStepDensity());
			principalPanel.add(getMinAgent());
			principalPanel.add(getMaxAgent());
			principalPanel.add(getStepAgent());
			principalPanel.add(getMinNbColors());
			principalPanel.add(getNbInstance());
			principalPanel.add(getFormat());
			principalPanel.add(getMaxNbColors());
			principalPanel.add(getStepNbColors());
			principalPanel.add(getNbVarsPerAgentLabel());
			principalPanel.add(getNbVarsPerAgent());
		}
		return principalPanel;
	}

	/**
	 * 
	 * @return JLabel
	 */
	private JLabel getNbVarsPerAgentLabel() {
		if (nbVarsPerAgentLabel == null) {
			nbVarsPerAgentLabel = new JLabel("Number of variables per agent", JLabel.CENTER);
			nbVarsPerAgentLabel.setBounds(new Rectangle(125, 270, 220, 22));
			nbVarsPerAgentLabel.setVisible(false);
		}
		return nbVarsPerAgentLabel;
	}

	/**
	 * This method initializes nbVarsPerAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbVarsPerAgent() {
		if (nbVarsPerAgent == null) {
			nbVarsPerAgent = new JSpinner(new SpinnerNumberModel(2, 1, 10, 1));
			nbVarsPerAgent.setBounds(new Rectangle(200, 300, 50, 22));
			nbVarsPerAgent.setVisible(false);
		}
		return nbVarsPerAgent;
	}

	/**
	 * This method initializes minDensity
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getMinDensity() {
		if (minDensity == null) {
			minDensity = new JSpinner(new SpinnerNumberModel(0.05, 0.01, 1, 0.01));
			minDensity.setBounds(new Rectangle(150, 180, 60, 22));
		}
		return minDensity;
	}

	/**
	 * This method initializes maxDensity
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getMaxDensity() {
		if (maxDensity == null) {
			maxDensity = new JSpinner(new SpinnerNumberModel(0.65, 0.01, 1, 0.01));
			maxDensity.setBounds(new Rectangle(220, 180, 60, 22));
		}
		return maxDensity;
	}

	/**
	 * This method initializes stepDensity
	 * 
	 * @return javax.swing.JTextField
	 */
	public JSpinner getStepDensity() {
		if (stepDensity == null) {
			stepDensity = new JSpinner(new SpinnerNumberModel(0.05, 0.01, 0.80, 0.01));
			stepDensity.setBounds(new Rectangle(290, 180, 60, 22));
		}
		return stepDensity;
	}

	private JSpinner getMinAgent() {
		if (minAgent == null) {
			minAgent = new JSpinner(new SpinnerNumberModel(10, 5, 20, 1));
			minAgent.setBounds(new Rectangle(150, 220, 60, 22));
		}
		return minAgent;
	}

	/**
	 * This method initializes maxAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxAgent() {
		if (maxAgent == null) {
			maxAgent = new JSpinner(new SpinnerNumberModel(30, 5, 1000, 1));
			maxAgent.setBounds(new Rectangle(220, 220, 60, 22));
		}
		return maxAgent;
	}

	/**
	 * This method initializes stepAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepAgent() {
		if (stepAgent == null) {
			stepAgent = new JSpinner(new SpinnerNumberModel(2, 1, 100, 1));
			stepAgent.setBounds(new Rectangle(290, 220, 60, 22));
		}
		return stepAgent;
	}

	/**
	 * This method initializes minNbColors
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinNbColors() {
		if (minNbColors == null) {
			minNbColors = new JSpinner(new SpinnerNumberModel(3, 2, 50, 1));
			minNbColors.setBounds(new Rectangle(150, 140, 60, 22));
		}
		return minNbColors;
	}

	/**
	 * This method initializes nbInstance
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbInstance() {
		if (nbInstance == null) {
			nbInstance = new JSpinner(new SpinnerNumberModel(25, 1, 500, 1));
			nbInstance.setBounds(new Rectangle(250, 60, 55, 22));
		}
		return nbInstance;
	}

	/**
	 * This method initializes format
	 * 
	 * @param format
	 */
	public void setFormat(String format) {
		if (format.equals(Commands.SIMPLE)) {
			getFormat().setSelectedItem(format);
		} else {
			if (format.equals(Commands.COMPLEX)) {
				getFormat().setSelectedItem(format);
			} else {
				System.err.println("Unkown format: " + format + " possible format are :" + Commands.SIMPLE + " and " + Commands.COMPLEX);
			}
		}
	}

	/**
	 * This method initializes format
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getFormat() {
		if (format == null) {
			format = new JComboBox();
			String[] formats = {Commands.SIMPLE, Commands.COMPLEX};
			format = new JComboBox(formats);
			format.setSelectedItem(Commands.SIMPLE);
			format.setBounds(new Rectangle(160, 20, 150, 25));
			format.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						String selected = (String) e.getItem();
						if (selected.equals(Commands.SIMPLE)) {
							getNbVarsPerAgentLabel().setVisible(false);
							getNbVarsPerAgent().setVisible(false);
						} else if (selected.equals(Commands.COMPLEX)) {
							getNbVarsPerAgentLabel().setVisible(true);
							// getNbVarsPerAgentLabel().revalidate();
							// getNbVarsPerAgentLabel().repaint();
							getNbVarsPerAgent().setVisible(true);
							// getNbVarsPerAgent().revalidate();
							// getNbVarsPerAgent().repaint();
						}
					}
				}
			});
		}
		return format;
	}

	/**
	 * This method initializes maxNbColors
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxNbColors() {
		if (maxNbColors == null) {
			maxNbColors = new JSpinner(new SpinnerNumberModel(3, 2, 50, 1));
			maxNbColors.setBounds(new Rectangle(220, 140, 60, 22));
		}
		return maxNbColors;
	}

	/**
	 * This method initializes stepNbColors
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepNbColors() {
		if (stepNbColors == null) {
			stepNbColors = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
			stepNbColors.setBounds(new Rectangle(290, 140, 60, 22));
		}
		return stepNbColors;
	}

	/**
	 * 
	 * @param file
	 */
	public void generate(File file) {
		int nbInstance = (Integer) (this.nbInstance.getValue());
		int minNbAgents = (Integer) (this.minAgent.getValue());
		int maxNbAgents = (Integer) (this.maxAgent.getValue());
		int stepNbAgents = (Integer) (this.stepAgent.getValue());
		int minNbColors = (Integer) (this.minNbColors.getValue());
		int maxNbColors = (Integer) (this.maxNbColors.getValue());
		int stepNbColors = (Integer) (this.stepNbColors.getValue());
		double minDensity = (Double) (this.minDensity.getValue());
		double maxDensity = (Double) (this.maxDensity.getValue());
		double stepDensity = (Double) (this.stepDensity.getValue());
		if (minDensity > 100 || maxDensity > 100 || stepDensity > 100 || minDensity < 0 || maxDensity < 0 || stepDensity < 0) {
			JOptionPane.showMessageDialog(this, "Set parameters correctly", "DisChoco", JOptionPane.WARNING_MESSAGE);
			return;
		}
		String pbFormat = (String) format.getSelectedItem();
		if (pbFormat.equals(Commands.SIMPLE)) {
			URGCGenerator gc = new URGCGenerator(file, nbInstance);
			gc.setNbNodes(minNbAgents, stepNbAgents, maxNbAgents);
			gc.setDensity(minDensity, stepDensity, maxDensity);
			gc.setNbCoulors(minNbColors, stepNbColors, maxNbColors);
			gc.generate();
		} else {
			// GraphColoringGenerator gc = new GraphColoringGenerator(file,
			// nbInstance, domainSize, pbFormat);
			// gc.setDensity(minDensity, maxDensity, stepDensity);
			// gc.setNumberAgents(minNbAgents, maxNbAgents, stepNbAgents);
			// if(pbFormat.equals(Commands.COMPLEX)){
			// int nbVars = (Integer) nbVarsPerAgent.getValue();
			// gc.setNumberVarsPerAgent(nbVars);
			// }
			// gc.generate();
		}
		file = null;
		JOptionPane.showMessageDialog(this, "Problems was generated with success");
		return;
	}

}
