/**
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with this program; if not, write to
 * the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * @version 2.0 DisChoco RandomDisCSPGeneratorPanel.java
 */
package benchmark.random.gui;

import static benchmark.random.RBDisCSPGenerator.GRAPH_DENSITY;
import static benchmark.random.RBDisCSPGenerator.TIGHTNESS;
import static util.parser.InstanceTokens.NB_AGENTS;
import static util.parser.InstanceTokens.NB_CONSTRAINTS;
import static util.parser.InstanceTokens.NB_TUPLES;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import dischoco.DisChoco;
import benchmark.random.RBDisCSPGenerator;
import gui.visualizing.Edge;
import gui.visualizing.Graph;
import gui.visualizing.GraphPanel;
import gui.visualizing.TreeDrawing;
import gui.visualizing.Vertex;
import kernel.order.AgentComplexity;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomDegAO;
import kernel.order.RandomAO;
import kernel.order.TreeAO;
import util.Commands;
import util.graph.PriorityTree;
import util.parser.InstanceParser;

/**
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 */

public class RBDisCSPPanel extends JPanel {

	private static final long serialVersionUID = -4353556871723921003L;
	private JLabel typeLabel = null;
	private JLabel prepocessingLabel = null;
	private JLabel randomDCSPLabel = null;
	private JLabel nbAgLabel = null;
	private JLabel nbAgentsLabel = null;
	private JLabel domLabel = null;
	private JLabel domainSizeLabel = null;
	private JLabel cstsdensityLabel = null;
	private JLabel P1Label = null;
	private JLabel cststightnessLabel = null;
	private JLabel P2Label = null;
	private JLabel nbCstsLabel = null;
	private JLabel nbConstrantsLabel = null;
	private JLabel tuplesLabel = null;
	private JLabel nbTuplesLabel = null;
	private JComboBox agentsOrdering = null;
	private JPanel eastPanel = null;
	private JPanel westPanel = null;
//	private File problem = null;
	private HashMap<Integer, AgentComplexity> complexities = null;
	private Graph<Vertex, Edge> graph = null;
	private JPanel graphPanel = null;
	private Dimension size = null;
	private final static String TreeAO_PANEL = "TreeAO_Panel";
	private final static String GraphAO_PANEL = "GraphAO_Panel";
	private int nbAgents;
	private int domainSize;
	private double p1;
	private double p2;
	private int nbConstraints;
	private int nbTuples;
	/**
	 * 
	 * @param file
	 * @param d
	 */
	public RBDisCSPPanel(Dimension d) {
		super();
		setSize(d);
		setPreferredSize(d);
		size = d;
		String fileName = DisChoco.filePath.getAbsolutePath();
		InstanceParser parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		complexities = parser.getAgentsComplexity();
		graph = parser.getGraph(new Dimension(650, 435));
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				System.out.println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try { document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(RBDisCSPGenerator.GuiPresentation);
		setPresentation(gui);
		setLayout(new BorderLayout());
		add(getWestPanel(), BorderLayout.WEST);
		add(getEastPanel(), BorderLayout.CENTER);
	}
	
	/**
	 * 
	 * @param width
	 * @param height
	 */
	public RBDisCSPPanel(int width, int height) {
		super();
		this.setSize(width, height);
	}

	public void setPresentation(Element presentation) {
		this.setLayout(null);
		this.nbAgents = new Integer(presentation.getAttributeValue(NB_AGENTS));
		this.domainSize = new Integer(presentation.getAttributeValue(Commands.DomainSize));
		this.p1 = new Double(presentation.getAttributeValue(GRAPH_DENSITY))/100;
		this.p2 = new Double(presentation.getAttributeValue(TIGHTNESS))/100;
		this.nbConstraints = new Integer(presentation.getAttributeValue(NB_CONSTRAINTS));
		this.nbTuples = new Integer(presentation.getAttributeValue(NB_TUPLES));
	}

	/**
	 * This method initializes westPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getWestPanel() {
		if (westPanel == null) {
			westPanel = new JPanel(null);
			Dimension d = new Dimension(350, size.height);
			westPanel.setSize(d);
			westPanel.setPreferredSize(d);
			typeLabel = new JLabel("Type :", JLabel.RIGHT);
			typeLabel.setBounds(new Rectangle(25, 100, 150, 22));
			prepocessingLabel = new JLabel("Prepocessing Ordering", JLabel.CENTER);
			prepocessingLabel.setBounds(new Rectangle(80, 15, 200, 22));
			randomDCSPLabel = new JLabel(Commands.RandomDisCSP, JLabel.LEFT);
			randomDCSPLabel.setBounds(new Rectangle(190, 100, 150, 22));
			nbAgLabel = new JLabel("Number of Agents :", JLabel.RIGHT);
			nbAgLabel.setBounds(new Rectangle(25, 135, 150, 22));
			nbAgentsLabel = new JLabel("", JLabel.LEFT);
			nbAgentsLabel.setBounds(new Rectangle(190, 135, 150, 22));
			domLabel = new JLabel("Domain size :", JLabel.RIGHT);
			domLabel.setBounds(new Rectangle(25, 170, 150, 22));
			domainSizeLabel = new JLabel("", JLabel.LEFT);
			domainSizeLabel.setBounds(new Rectangle(190, 170, 150, 22));
			cstsdensityLabel = new JLabel("constraint density :", JLabel.RIGHT);
			cstsdensityLabel.setBounds(new Rectangle(25, 205, 150, 22));
			P1Label = new JLabel("", JLabel.LEFT);
			P1Label.setBounds(new Rectangle(190, 205, 150, 22));
			cststightnessLabel = new JLabel("constraints tightness :", JLabel.RIGHT);
			cststightnessLabel.setBounds(new Rectangle(25, 240, 150, 22));
			P2Label = new JLabel("", JLabel.LEFT);
			P2Label.setBounds(new Rectangle(190, 240, 150, 22));
			nbCstsLabel = new JLabel("Number of constraints :", JLabel.RIGHT);
			nbCstsLabel.setBounds(new Rectangle(25, 275, 150, 22));
			nbConstrantsLabel = new JLabel("", JLabel.LEFT);
			nbConstrantsLabel.setBounds(new Rectangle(190, 275, 150, 22));
			tuplesLabel = new JLabel("Number of tuples :", JLabel.RIGHT);
			tuplesLabel.setBounds(new Rectangle(25, 310, 150, 22));
			nbTuplesLabel = new JLabel("", JLabel.LEFT);
			nbTuplesLabel.setBounds(new Rectangle(190, 310, 150, 22));
			westPanel.add(prepocessingLabel);
			westPanel.add(getAgentsOrdering());
			westPanel.add(typeLabel);
			westPanel.add(randomDCSPLabel);
			westPanel.add(nbAgLabel);
			westPanel.add(nbAgentsLabel);
			westPanel.add(domLabel);
			westPanel.add(domainSizeLabel);
			westPanel.add(cstsdensityLabel);
			westPanel.add(cststightnessLabel);
			westPanel.add(P1Label);
			westPanel.add(P2Label);
			westPanel.add(nbCstsLabel);
			westPanel.add(nbConstrantsLabel);
			westPanel.add(tuplesLabel);
			westPanel.add(nbTuplesLabel);
			nbAgentsLabel.setText(nbAgents + "");
			domainSizeLabel.setText(domainSize + "");
			P1Label.setText(p1 + "");
			P2Label.setText(p2 + "");
			nbConstrantsLabel.setText(nbConstraints + "");
			nbTuplesLabel.setText(nbTuples + "");
		}
		return westPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			agentsOrdering = new JComboBox();
			agentsOrdering.setBounds(new Rectangle(100, 50, 150, 22));
			agentsOrdering.addItem(Commands.LexicoAO);
			agentsOrdering.addItem(Commands.RandomAO);
			agentsOrdering.addItem(Commands.MinDomDegAO);
			agentsOrdering.addItem(Commands.MaxDegPlusMinDomAO);
			agentsOrdering.addItem(Commands.TreeAO);
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						plotEastPanel((String) e.getItem());
					}
				}
			});
			System.out.println("Agent ordering : " + LexicographicAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
		return agentsOrdering;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel(new CardLayout());
			Dimension d = new Dimension(size.width-350, size.height);
			eastPanel.setSize(d);
			eastPanel.setPreferredSize(d);
		}
		return eastPanel;
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getGrpahPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel(graph, new Dimension(size.width-350, size.height));
		}
		return graphPanel;
	}

	private void plotEastPanel(String selectedItem) {
		if (selectedItem.equals(Commands.TreeAO)) {
			plotTreeAO();
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			System.out.println("Agent ordering : " + LexicographicAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.RandomAO)) {
			System.out.println("Agent ordering : " + RandomAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MinDomDegAO)) {
			System.out.println("Agent ordering : " + MinDomDegAO.buildpriorityTree(complexities));
			plotGraphAO();
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			System.out.println("Agent ordering : " + MaxDegPlusMinDomAO.buildpriorityTree(complexities));
			plotGraphAO();
		}
	}

	/**
	 * 
	 */
	private void plotGraphAO() {
		getEastPanel().add(getGrpahPanel(), GraphAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), GraphAO_PANEL);
	}

	private void plotTreeAO() {
		PriorityTree tree = TreeAO.buildpriorityTree(this.complexities);
		System.out.println("Agent ordering (BFS) : [" + tree.toBFSString()+"]");
		tree.position(0);
		getEastPanel().add(new TreeDrawing(tree, getEastPanel().getSize()), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getEastPanel().getLayout());
		cards.show(getEastPanel(), TreeAO_PANEL);
	}
}
