package benchmark.rmsp;
/**
 * 
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class Place implements Comparable<Object> {
	
	private int id;
	private int x;
	private int y;
	private String name;

	public Place(int id) {
		super();
		this.id = id;
		this.name = "";
	}

	public Place(int id, int x, int y) {
		this(id);
		setLocation(x, y);
	}

	public Place(int id, int x, int y, String name) {
		this(id, x, y);
		this.name = name;
	}

	public int getID() {
		return this.id;
	}

	public int getX() {
		return this.x;
	}

	public int getY() {
		return this.y;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return this.name;
	}

	public void setLocation(int x, int y) {
		this.x = x;
		this.y = y;
	}

	public String toStringX() {
		return String.valueOf(this.x);
	}

	public String toStringY() {
		return String.valueOf(this.y);
	}

	public String pretty() {
		return this.name+"("+this.id+")"+ " = (" + this.x + "," + this.y + ")";
	}

	public String toString() {
		return this.id+"=(" + this.x + "," + this.y + ")";
	}

	public static Place fromString(String place) {
		int id = new Integer(place.substring(0, place.indexOf("=")));
		place = place.substring(place.indexOf("(") + 1, place.indexOf(")"));
		String[] p = place.split(",");
		int x = new Integer(p[0]);
		int y = new Integer(p[1]);
		return new Place(id, x, y);//TODO
	}

	public int compareTo(Object obj) {
		Place arg = (Place) obj;
		if (this.x > arg.x || (this.x == arg.x && this.y > arg.y)) {
			return 1;
		} else if (this.x == arg.x && this.y == arg.y) {
			return 0;
		} else {
			return -1;
		}
	}

	/**
	 * Returns the distance between two points.
	 * 
	 * @param x1
	 *            the X coordinate of the first specified place
	 * @param y1
	 *            the Y coordinate of the first specified place
	 * @param x2
	 *            the X coordinate of the second specified place
	 * @param y2
	 *            the Y coordinate of the second specified place
	 * @return the distance between the two sets of specified coordinates.
	 * @since 1.2
	 */
	public static double distance(int x1, int y1, double x2, double y2) {
		x1 -= x2;
		y1 -= y2;
		return Math.sqrt(x1 * x1 + y1 * y1);
	}

	/**
	 * Returns the square of the distance from this <code>Place</code> to a specified place.
	 * 
	 * @param px
	 *            the X coordinate of the specified place to be measured against this <code>Place</code>
	 * @param py
	 *            the Y coordinate of the specified place to be measured against this <code>Place</code>
	 * @return the square of the distance between this <code>Place</code> and the specified place.
	 */
	public double distanceSq(int px, int py) {
		px -= this.getX();
		py -= this.getY();
		return (px * px + py * py);
	}

	/**
	 * Returns the square of the distance from this <code>Place</code> to a specified <code>Place</code>.
	 * 
	 * @param place
	 *            the specified place to be measured against this <code>Place</code>
	 * @return the square of the distance between this <code>Place</code> to a specified <code>Place</code>.
	 */
	public double distanceSq(Place place) {
		int px = place.getX() - this.getX();
		int py = place.getY() - this.getY();
		return (px * px + py * py);
	}

	/**
	 * Returns the distance from this <code>Place</code> to a specified <code>Place</code>.
	 * 
	 * @param place
	 *            the specified place to be measured against this <code>Place</code>
	 * @return the distance between this <code>Place</code> and the specified <code>Place</code>.
	 */
	public double distance(Place place) {
		int px = place.getX() - this.getX();
		int py = place.getY() - this.getY();
		return Math.sqrt(px * px + py * py);
	}

	/**
	 * Determines whether or not two places are equal. Two instances of <code>Place</code> are equal if the values of their <code>x</code> and
	 * <code>y</code> member fields, representing their position in the coordinate space, are the same.
	 * 
	 * @param obj
	 *            an object to be compared with this <code>Place</code>
	 * @return <code>true</code> if the object to be compared is an instance of <code>Place</code> and has the same values; <code>false</code>
	 *         otherwise.
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Place) {
			Place place = (Place) obj;
			return (this.getX() == place.getX()) && (this.getY() == place.getY());
		}
		return super.equals(obj);
	}
}
