package benchmark.rmsp.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.File;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import benchmark.rmsp.RandomMSP;
import gui.generators.Generator;
import static gui.Gui.*;

public class RandomMSPGeneratorPanel extends Generator {

	private static final long serialVersionUID = -3098482692945226862L;
	
	public static final String fixKeys = P_V;
	
	private JPanel principalPanel = null;
	public JLabel nbInstanceLabel = null;
	public JLabel nbAgentLabel = null;
	public JLabel meetingPerAgentLabel = null;
	public JLabel daysLabel = null;
	public JLabel hoursLabel = null;
	private JLabel nbOfMeetingLabel = null;
	private JLabel durationLabel = null;
	private JLabel gridSizeLabel = null;
	private JSpinner nbInstance = null;
	private JSpinner minNbAgents = null;
	private JSpinner minNbOfMeeting = null;
	private JSpinner minMeetingPerAgent = null;
	private JSpinner nbDays = null;
	private JSpinner nbHours = null;
	private JSpinner duration = null;
	private JSpinner minGridSize = null;
	private JSpinner maxGridSize = null;
	private JSpinner stepGridSize = null;
	private JSpinner maxNbAgents = null;
	private JLabel minLabel = null;
	private JLabel maxLabel = null;
	private JLabel stepLabel = null;
	private JSpinner stepNbAgents = null;
	private JLabel disponibilityLabel = null;
	private JSpinner disponibility = null;
	private JLabel percentLabel = null;
	private JSpinner maxMeetingPerAgent = null;
	private JSpinner stepMeetingPerAgent = null;
	private JSpinner maxNbOfMeeting = null;
	private JSpinner stepNbOfMeeting = null;

	/**
	 * This is the default constructor
	 */
	public RandomMSPGeneratorPanel() {
		super();
		this.add(this.getPrincipalPanel(), BorderLayout.CENTER);
	}

	protected JPanel getPrincipalPanel() {
		if (principalPanel == null) {
			principalPanel = new JPanel();
			principalPanel.setLayout(null);
			principalPanel.setPreferredSize(new Dimension(575, 375));
			principalPanel.setMinimumSize(new Dimension(575, 375));
			principalPanel.add(getNbInstances(), null);
			principalPanel.add(nbInstanceLabel, null);
			principalPanel.add(getNbDays(), null);
			principalPanel.add(daysLabel, null);
			principalPanel.add(getNbHours(), null);
			principalPanel.add(hoursLabel, null);
			principalPanel.add(getDuration(), null);
			principalPanel.add(durationLabel, null);
			principalPanel.add(getMinGridSize(), null);
			principalPanel.add(gridSizeLabel, null);
			principalPanel.add(getMaxGridSize(), null);
			principalPanel.add(getMinNbAgents(), null);
			principalPanel.add(getStepGridSize(), null);
			principalPanel.add(nbAgentLabel, null);
			principalPanel.add(getMaxNbAgents(), null);
			principalPanel.add(getStepNbAgents(), null);
			principalPanel.add(getDisponibility(), null);
			principalPanel.add(disponibilityLabel, null);
			principalPanel.add(percentLabel, null);
			minLabel = new JLabel();
			minLabel.setBounds(new Rectangle(225, 105, 80, 22));
			minLabel.setText(MINIMUM);
			minLabel.setHorizontalAlignment(JLabel.CENTER);
			principalPanel.add(minLabel, null);
			maxLabel = new JLabel();
			maxLabel.setBounds(new Rectangle(330, 105, 80, 22));
			maxLabel.setText(MAXIMUM);
			maxLabel.setHorizontalAlignment(JLabel.CENTER);
			principalPanel.add(maxLabel, null);
			stepLabel = new JLabel();
			stepLabel.setBounds(new Rectangle(430, 105, 80, 22));
			stepLabel.setText(STEP);
			stepLabel.setHorizontalAlignment(JLabel.CENTER);
			principalPanel.add(stepLabel, null);
			principalPanel.add(getMinMeetingPerAgent(), null);
			principalPanel.add(getMaxMeetingPerAgent(), null);
			principalPanel.add(getStepMeetingPerAgent(), null);
			principalPanel.add(getMinNbMeetings(), null);
			principalPanel.add(getMaxNbOfMeeting(), null);
			principalPanel.add(getStepNbOfMeeting(), null);
			principalPanel.add(nbOfMeetingLabel, null);
			principalPanel.add(meetingPerAgentLabel, null);
		}
		return principalPanel;
	}

	public void generate(File directory) {
		// if( ){
		// JOptionPane.showMessageDialog(this, "Set parameters correctly",
		// "DisChoco", JOptionPane.WARNING_MESSAGE);
		// return;
		// }
		int nbInstances = (Integer) (getNbInstances().getValue());
		int minNbMeetings = (Integer) (getMinNbMeetings().getValue());
		int maxNbMeetings = (Integer) (getMaxNbOfMeeting().getValue());
		int stepNbMeetings = (Integer) (getStepNbOfMeeting().getValue());

		int minNbAgents = (Integer) (getMinNbAgents().getValue());
		int maxNbAgents = (Integer) (getMaxNbAgents().getValue());
		int stepNbAgents = (Integer) (getStepNbAgents().getValue());

		int minMeetingsPerAgent = (Integer) (getMinMeetingPerAgent().getValue());
		int maxMeetingsPerAgent = (Integer) (getMaxMeetingPerAgent().getValue());
		int stepMeetingsPerAgent = (Integer) (getStepMeetingPerAgent().getValue());

		int duration = (Integer) (getDuration().getValue());
		int nbDays = (Integer) (getNbDays().getValue());
		int nbHours = (Integer) (getNbHours().getValue());
		int minGridSize = (Integer) (getMinGridSize().getValue());
		int maxGridSize = (Integer) (getMaxGridSize().getValue());
		int stepGridSize = (Integer) (getStepGridSize().getValue());
		int disponibility = (Integer) (getDisponibility().getValue());
		RandomMSP msp = new RandomMSP(directory, nbInstances, duration);
		msp.setMeetingsProperties(minNbMeetings, maxNbMeetings, stepNbMeetings);
		msp.setAgentsProperties(minNbAgents, maxNbAgents, stepNbAgents);
		msp.setMeetingsPerAgentsProperties(minMeetingsPerAgent, maxMeetingsPerAgent, stepMeetingsPerAgent);
		msp.setGeographyProperties(minGridSize, maxGridSize, stepGridSize);
		msp.setCalendarProperties(nbDays, nbHours, disponibility);
		msp.generate();
		directory = null;
		JOptionPane.showMessageDialog(this, "Problems was generated with success");
		return;
	}

	/**
	 * This method initializes nbInstance
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbInstances() {
		if (nbInstance == null) {
			nbInstance = new JSpinner(new SpinnerNumberModel(25, 1, 200, 1));
			nbInstance.setBounds(new Rectangle(220, 25, 60, 24));
			nbInstanceLabel = new JLabel();
			nbInstanceLabel.setBounds(new Rectangle(60, 25, 150, 22));
			nbInstanceLabel.setText(NumberOfInstances);
			nbInstanceLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return nbInstance;
	}

	/**
	 * This method initializes minNbAgents
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinNbAgents() {
		if (minNbAgents == null) {
			minNbAgents = new JSpinner(new SpinnerNumberModel(16, 3, 50, 1));
			minNbAgents.setBounds(new Rectangle(235, 175, 60, 24));
			nbAgentLabel = new JLabel();
			nbAgentLabel.setBounds(new Rectangle(80, 175, 130, 22));
			nbAgentLabel.setText(NumberOfAgents);
			nbAgentLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return minNbAgents;
	}

	/**
	 * This method initializes nbOfMeeting
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinNbMeetings() {
		if (minNbOfMeeting == null) {
			minNbOfMeeting = new JSpinner(new SpinnerNumberModel(9, 5, 50, 1));
			minNbOfMeeting.setBounds(new Rectangle(235, 135, 60, 24));
			nbOfMeetingLabel = new JLabel();
			nbOfMeetingLabel.setBounds(new Rectangle(80, 135, 130, 22));
			nbOfMeetingLabel.setText(NumberOfMeetings);
			nbOfMeetingLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return minNbOfMeeting;
	}

	/**
	 * This method initializes minMeetingPerAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinMeetingPerAgent() {
		if (minMeetingPerAgent == null) {
			minMeetingPerAgent = new JSpinner(new SpinnerNumberModel(2, 2, 8, 1));
			minMeetingPerAgent.setBounds(new Rectangle(235, 215, 60, 22));
			meetingPerAgentLabel = new JLabel();
			meetingPerAgentLabel.setBounds(new Rectangle(10, 215, 200, 22));
			meetingPerAgentLabel.setText(NumberOfMeetingsPerAgent);
			meetingPerAgentLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return minMeetingPerAgent;
	}

	/**
	 * This method initializes nbDays
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbDays() {
		if (nbDays == null) {
			nbDays = new JSpinner(new SpinnerNumberModel(2, 1, 30, 1));
			nbDays.setBounds(new Rectangle(220, 65, 60, 22));
			daysLabel = new JLabel();
			daysLabel.setBounds(new Rectangle(60, 65, 150, 22));
			daysLabel.setText(NumberOfDays);
			daysLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return nbDays;
	}

	/**
	 * This method initializes nbHours
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getNbHours() {
		if (nbHours == null) {
			nbHours = new JSpinner(new SpinnerNumberModel(10, 2, 24, 1));
			nbHours.setBounds(new Rectangle(490, 65, 60, 24));
			hoursLabel = new JLabel();
			hoursLabel.setBounds(new Rectangle(330, 65, 150, 22));
			hoursLabel.setText(NumberOfHours);
			hoursLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return nbHours;
	}

	/**
	 * This method initializes duration
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getDuration() {
		if (duration == null) {
			duration = new JSpinner(new SpinnerNumberModel(1, 1, 8, 1));
			duration.setBounds(new Rectangle(490, 25, 60, 24));
			durationLabel = new JLabel();
			durationLabel.setBounds(new Rectangle(330, 25, 150, 22));
			durationLabel.setText(DurationOfMeetings);
			durationLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return duration;
	}

	/**
	 * This method initializes minGridSize
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMinGridSize() {
		if (minGridSize == null) {
			minGridSize = new JSpinner(new SpinnerNumberModel(3, 1, 1000, 1));
			minGridSize.setBounds(new Rectangle(235, 255, 60, 24));
			gridSizeLabel = new JLabel();
			gridSizeLabel.setBounds(new Rectangle(60, 255, 150, 22));
			gridSizeLabel.setText("Grid sizes");
			gridSizeLabel.setHorizontalAlignment(JLabel.RIGHT);
		}
		return minGridSize;
	}

	/**
	 * This method initializes maxGridSize
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxGridSize() {
		if (maxGridSize == null) {
			maxGridSize = new JSpinner(new SpinnerNumberModel(21, 1, 2000, 1));
			maxGridSize.setBounds(new Rectangle(340, 255, 60, 24));
		}
		return maxGridSize;
	}

	/**
	 * This method initializes maxGridSize
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepGridSize() {
		if (stepGridSize == null) {
			stepGridSize = new JSpinner(new SpinnerNumberModel(2, 1, 2000, 1));
			stepGridSize.setBounds(new Rectangle(440, 255, 60, 24));
		}
		return stepGridSize;
	}

	/**
	 * This method initializes maxNbAgents
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxNbAgents() {
		if (maxNbAgents == null) {
			maxNbAgents = new JSpinner(new SpinnerNumberModel(20, 5, 50, 1));
			maxNbAgents.setBounds(new Rectangle(340, 175, 60, 22));
		}
		return maxNbAgents;
	}

	/**
	 * This method initializes stepNbAgents
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepNbAgents() {
		if (stepNbAgents == null) {
			stepNbAgents = new JSpinner(new SpinnerNumberModel(2, 0, 25, 1));
			stepNbAgents.setBounds(new Rectangle(440, 175, 60, 24));
		}
		return stepNbAgents;
	}

	/**
	 * This method initializes disponibility
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getDisponibility() {
		if (disponibility == null) {
			disponibility = new JSpinner(new SpinnerNumberModel(100, 1, 100, 1));
			disponibility.setBounds(new Rectangle(340, 300, 60, 24));
			disponibilityLabel = new JLabel();
			disponibilityLabel.setBounds(new Rectangle(180, 300, 150, 22));
			disponibilityLabel.setText(DisponibilityOfAgents);
			disponibilityLabel.setHorizontalAlignment(JLabel.RIGHT);
			percentLabel = new JLabel();
			percentLabel.setBounds(new Rectangle(405, 300, 20, 22));
			percentLabel.setText("%");
			percentLabel.setHorizontalAlignment(JLabel.LEFT);
		}
		return disponibility;
	}

	/**
	 * This method initializes maxMeetingPerAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxMeetingPerAgent() {
		if (maxMeetingPerAgent == null) {
			maxMeetingPerAgent = new JSpinner(new SpinnerNumberModel(7, 2, 20, 1));
			maxMeetingPerAgent.setBounds(new Rectangle(340, 215, 60, 22));
		}
		return maxMeetingPerAgent;
	}

	/**
	 * This method initializes stepMeetingPerAgent
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepMeetingPerAgent() {
		if (stepMeetingPerAgent == null) {
			stepMeetingPerAgent = new JSpinner(new SpinnerNumberModel(1, 0, 25, 1));
			stepMeetingPerAgent.setBounds(new Rectangle(440, 215, 60, 22));
		}
		return stepMeetingPerAgent;
	}

	/**
	 * This method initializes maxNbOfMeeting
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getMaxNbOfMeeting() {
		if (maxNbOfMeeting == null) {
			maxNbOfMeeting = new JSpinner(new SpinnerNumberModel(11, 5, 50, 1));
			maxNbOfMeeting.setBounds(new Rectangle(340, 135, 60, 22));
		}
		return maxNbOfMeeting;
	}

	/**
	 * This method initializes stepNbOfMeeting
	 * 
	 * @return javax.swing.JSpinner
	 */
	private JSpinner getStepNbOfMeeting() {
		if (stepNbOfMeeting == null) {
			stepNbOfMeeting = new JSpinner(new SpinnerNumberModel(1, 0, 10, 1));
			stepNbOfMeeting.setBounds(new Rectangle(440, 135, 60, 22));
		}
		return stepNbOfMeeting;
	}

}
