package dischoco;

import java.util.Iterator;
import java.util.SortedMap;
import java.util.TreeMap;

import kernel.AbstractMaster;
import simple.dcsp.abtfamily.abt.ABTMaster;
import simple.dcsp.abtfamily.abt.abtac.ABTacMaster;
import simple.dcsp.abtfamily.abtdac.ABTdacMaster;
import simple.dcsp.abtfamily.abtdo.ABTDOMaster;
import simple.dcsp.abtfamily.abtdoretro.ABTDORetroMaster;
import simple.dcsp.abtfamily.abtdoretroz.ZABTDORetroMaster;
import simple.dcsp.abtfamily.abthyb.ABTHybMaster;
import simple.dcsp.afcfamily.afc.AFCMaster;
import simple.dcsp.afcfamily.afc.domdeg.AFCDomDegMaster;
import simple.dcsp.afcfamily.afc.domdeg.AFCMinDomMaster;
import simple.dcsp.afcfamily.afc.estimDom.AFCEstimatedMaster;
import simple.dcsp.afcfamily.afc.triggerednogood.AFCTriggeredNogoodMaster;
import simple.dcsp.afcfamily.afc_dbj.AFCDBJMaster;
import simple.dcsp.afcfamily.afcng.AFCngMaster;
import simple.dcsp.afcfamily.afcng.afctree.AFCTreeMaster;
import simple.dcsp.afcfamily.afcng.approximateDom.AFCngApproximateMinDomMaster;
import simple.dcsp.afcfamily.afcng.domDeg.AFCngDomDegMaster;
import simple.dcsp.afcfamily.afcng.domDeg.AFCngMinDomMaster;
import simple.dcsp.afcfamily.afcng.triggerednogood.AFCngTriggeredNogoodMaster;
import simple.dcsp.afcfamily.ddbj.DDBJMaster;
//import simple.dcsp.agile.exp.AgileMaster;
//import simple.dcsp.agileabt.explanation.AgileABTMaster;
import simple.dcsp.dba.DBAmaster;
import simple.dcsp.sbt.SBTMaster;

/**
 * 
 * 
 * @authors <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr"> Redoaune Ezzahir</a> and <a
 *          href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr"> Mohamed Wahbi</a>
 * 
 */
public final class Protocols {

	public static final String SBT = "SBT";
	/** ABT Bessiere et al. 2005 */
	public static final String ABT = "ABT";
	/**  */
	public static final String ABTHyb = "ABT-hyb";
	/**  */
	public static final String ABTdac = "ABTdac";
	/**  */
	public static final String ABTac = "ABTac";

	/** ABT with Dynamic Ordering Family Algorithms */
	public static final String ABTDOng = "ABTDO-ng";
	/**  */
	public static final String ABTDOMinDom = "ABTDOMinDom";
	/**  */
	public static final String ABTDORandom = "ABTDORandom";
	/**  */
	public static final String ABTDOMinDomS = "ABTDOMinDom-S";
	/**  */
	public static final String ABTDORandomS = "ABTDORandom-S";

	/** MinDomain retroactive Heuristics for ABT */
	public static final String ABTDORetro = "ABTDORetro";
	/** Minimum Domain */
	public static final String ABTDORetroMinDom = "ABTDORetroMinDom";
	/** After Second Last */
	public static final String ABTDORetroA2ndL = "ABTDORetroA2ndL"; 
	/** Domain Position Single */
	public static final String ABTDORetroDomPS = "ABTDORetroDomPS";
	/** Domain Position All */
	public static final String ABTDORetroDomPAll = "ABTDORetroDomPAll";
	/** Highest */
	public static final String ABTDORetroHighest = "ABTDORetroHighest";
	
	public static final String AFC = "AFC";
	public static final String AFCTriggeredng = "AFCTriggeredng";
	public static final String AFCMinDom = "AFCMinDom";
	public static final String AFCDomDeg = "AFCDomDeg";
	public static final String AFCEstMinDom = "AFCEstMinDom";
	public static final String AFCEstDomDeg = "AFCEstDomDeg";

	/** Dynamic Distributed BackJumping: Viet Nguyen, Djamila Sam-Haroud, and Boi Faltings*/
	public static final String AFCdbj = "AFC-dbj";
	public static final String DDBJ = "DDBJ";	
	
	public static final String AFCNR = "AFC-nr";

	public static final String AFCng = "AFC-ng";

	public static final String AFCng2 = "AFC-ng2";
	public static final String AFCngTriggeredng = "AFC-ng-TriggeredNogood";
	public static final String AFCngMinDom = "AFC-ng-MinDom";
	public static final String AFCngDomDeg = "AFC-ng-DomDeg";
	public static final String AFCngMaxDeg = "AFC-ng-MaxDeg";
	public static final String AFCngApproximatedMinDom = "AFC-ng-ApproximatedMinDom";
	public static final String AFCngApproximatedDomDeg = "AFC-ng-ApproximatedDomDeg";


	public static final String AFCtree = "AFC-tree";
	
	//====================================================================
//	public static final String AFCac0 = "AFCAC0";
//	public static final String AFCac1 = "AFCAC1";
//	public static final String AFCac2 = "AFCAC2";
//	public static final String AFCac3 = "AFCAC3";
//	
//	public static final String AFCNRCM0 = "AFCNR-CM0";
//	public static final String AFCNRCM1 = "AFCNR-CM1";
//	public static final String AFCNRCM2 = "AFCNR-CM2";
//	public static final String AFCNRCM3 = "AFCNR-CM3";
//
//	public static final String AFCngAC0 = "AFCngAC0";
//	public static final String AFCngAC1 = "AFCngAC1";
//	public static final String AFCngAC2 = "AFCngAC2";
//	public static final String AFCngAC3 = "AFCngAC3";
//	
//	public static final String AFCtreeAC0 = "AFCtreeAC0";
//	public static final String AFCtreeAC1 = "AFCtreeAC1";
//	public static final String AFCtreeAC2 = "AFCtreeAC2";
//	public static final String AFCtreeAC3 = "AFCtreeAC3";
	//====================================================================

	public static final String DisFC = "DisFC";
	public static final String DisFC1ph = "DisFC1ph";
	public static final String DisFC2ph = "DisFC2ph";
	public static final String DisFClies = "DisFClies";

	public static final String Adopt = "Adopt";
	public static final String BnBAdopt = "BnBAdopt";
	public static final String AFB = "AFB";

	public static final String DBA = "DBA";

	public static final String ABTcf = "ABTcf";

	public static final String SDisCSP = "SDisCSP";
	public static final String SDCOP = "SDCOP";
	public static final String CDisCSP = "CDisCSP";
	public static final String CDCOP = "CDCOP";
	
	/** Agile ABT Family : best heuristic sending only in */
	public static final String AgileMinDom = "AgileABTMinDom";
	public static final String AgileDomDeg = "AgileABTDomDeg";
	public static final String Agile = AgileMinDom;
	public static final String Agile_ALL = "AgileABT-All";
	public static final String Agile_O = "AgileABT-O";
	public static final String Agile_N = "AgileABT-N";
	public static final String Agile_ON = "AgileABT-ON";
	public static final String AgileMinDom_ID = "AgileABTMinDomTopoID";
	public static final String AgileMinDom_Deg = "AgileABTMinDomTopoDeg";
	public static final String AgileMinDom_MaxSucc = "AgileABTMinDomTopoMaxSucc";
	public static final String AgileDomDeg_ID = "AgileABTDomDegTopoID";
	public static final String AgileDomDeg_Deg = "AgileABTDomDegTopoDeg";
	public static final String AgileDomDeg_MaxSucc = "AgileABTDomDegTopoMaxSucc";
	
	public static final String Agiledom 			= "AgileABTdom";
	public static final String Agiledom_ID 			= "AgileABTdom+id";
	public static final String Agiledom_Deg 		= "AgileABTdom+deg";
	public static final String Agiledom_MaxSucc 	= "AgileABTdom+maxSucc";
	
	public static final String Agiledomdeg 			= "AgileABTdomdeg";
	public static final String Agiledomdeg_ID 		= "AgileABTdomdeg+id";
	public static final String Agiledomdeg_Deg 		= "AgileABTdomdeg+deg";
	public static final String Agiledomdeg_MaxSucc 	= "AgileABTdomdeg+maxSucc";
	
	public static final String Agiledomwdeg 		= "AgileABTdomwdeg";
	public static final String Agiledomwdeg_ID 		= "AgileABTdomwdeg+id";
	public static final String Agiledomwdeg_Deg 	= "AgileABTdomwdeg+deg";
	public static final String Agiledomwdeg_MaxSucc = "AgileABTdomwdeg+maxSucc";
	
	/** MinDomain Retroactive Heuristics for ABT **/
	public static final String ZABTDORetro = "ZABTDORetro";
	/** Minimum Domain */
	public static final String ZABTDORetroMinDom = "ZABTDORetroMinDom";
	/** After Second Last */
	public static final String ZABTDORetroA2ndL = "ZABTDORetroA2ndL"; 
	/** Domain Position Single */
	public static final String ZABTDORetroDomPS = "ZABTDORetroDomPS";
	/** Domain Position All */
	public static final String ZABTDORetroDomPAll = "ZABTDORetroDomPAll";
	/** Highest */
	public static final String ZABTDORetroHighest = "ZABTDORetroHighest";

	private static final SortedMap<String, Integer> protocoles = new TreeMap<String, Integer>();

	static {
		protocoles.put(SBT, 0);

		protocoles.put(ABT, 		1);
		protocoles.put(ABTHyb, 		2);
		protocoles.put(ABTac, 		3);
		protocoles.put(ABTdac, 		4);

		/* ===================================================================================================== */
		protocoles.put(AFC, 			101); 
		protocoles.put(AFCMinDom,		102);
		protocoles.put(AFCDomDeg, 		103);
		protocoles.put(AFCTriggeredng, 	104);
		protocoles.put(AFCEstMinDom, 	105);
		protocoles.put(AFCEstDomDeg, 	106);
		//------------------------------------------------------------------

		protocoles.put(AFCdbj, 	108);
		protocoles.put(DDBJ, 	109);
		/* ===================================================================================================== */
//		protocoles.put(AFCac0, 			111);
//		protocoles.put(AFCac1, 			112);
//		protocoles.put(AFCac2, 			113);
//		protocoles.put(AFCac3, 			114);
		/* ===================================================================================================== */
		protocoles.put(AFCNR, 			121);
		/* ===================================================================================================== */
//		protocoles.put(AFCNRCM0, 		132);
//		protocoles.put(AFCNRCM1, 		133);
//		protocoles.put(AFCNRCM2, 		134);
//		protocoles.put(AFCNRCM3, 		135);
		/* ===================================================================================================== */
//		protocoles.put(AFCng2, 			140);
		protocoles.put(AFCng, 			141);
		protocoles.put(AFCngMinDom, 	142);
		protocoles.put(AFCngDomDeg, 	143);
		protocoles.put(AFCngTriggeredng,144);
		protocoles.put(AFCngApproximatedMinDom, 145);
		protocoles.put(AFCngApproximatedDomDeg, 146);
		/* ===================================================================================================== */
//		protocoles.put(AFCngAC0, 		151);
//		protocoles.put(AFCngAC1,  		152);
//		protocoles.put(AFCngAC2,  		153);
//		protocoles.put(AFCngAC3,  		154);
		/* ===================================================================================================== */
		protocoles.put(AFCtree, 		161);
		/* ===================================================================================================== */
//		protocoles.put(AFCtreeAC0, 		162);
//		protocoles.put(AFCtreeAC1, 		163);
//		protocoles.put(AFCtreeAC2, 		164);
//		protocoles.put(AFCtreeAC3, 		165);
		/* ===================================================================================================== */
		protocoles.put(ABTDOng, 			211);
		protocoles.put(ABTDOMinDom, 		212);
		protocoles.put(ABTDORandom, 		213);
		protocoles.put(ABTDOMinDomS, 		214);
		protocoles.put(ABTDORandomS, 		215);
		/* ===================================================================================================== */
		protocoles.put(ABTDORetro, 			221);
		protocoles.put(ABTDORetroMinDom,	222);
		protocoles.put(ABTDORetroA2ndL, 	223);
		protocoles.put(ABTDORetroDomPS, 	224);
		protocoles.put(ABTDORetroDomPAll,	225);
		protocoles.put(ABTDORetroHighest, 	226);
		/* ===================================================================================================== */
		/* ===================================================================================================== */
		protocoles.put(ABTDOng, 			211);
		protocoles.put(ABTDOMinDom, 		212);
		protocoles.put(ABTDORandom, 		213);
		protocoles.put(ABTDOMinDomS, 		214);
		protocoles.put(ABTDORandomS, 		215);
		/* ===================================================================================================== */
		protocoles.put(ABTDORetro, 			221);
		protocoles.put(ABTDORetroMinDom,	222);
		protocoles.put(ABTDORetroA2ndL, 	223);
		protocoles.put(ABTDORetroDomPS, 	224);
		protocoles.put(ABTDORetroDomPAll,	225);
		protocoles.put(ABTDORetroHighest, 	226);
		/* ===================================================================================================== */
		protocoles.put(AgileMinDom, 		231);
		protocoles.put(Agile_ALL, 			232);
		protocoles.put(Agile_O,				233);
		protocoles.put(Agile_N, 			234);
		protocoles.put(Agile_ON, 			235);
		protocoles.put(AgileMinDom_ID, 		236);
		protocoles.put(AgileMinDom_Deg, 	237);
		protocoles.put(AgileMinDom_MaxSucc, 238);
		protocoles.put(AgileDomDeg, 		241);
		protocoles.put(AgileDomDeg_ID, 		242);
		protocoles.put(AgileDomDeg_Deg, 	243);
		protocoles.put(AgileMinDom_MaxSucc, 244);
		/* ===================================================================================================== */
		protocoles.put(Agiledom, 			251);
		protocoles.put(Agiledom_ID, 		252);
		protocoles.put(Agiledom_Deg, 		253);
		protocoles.put(Agiledom_MaxSucc, 	254);
		
		protocoles.put(Agiledomdeg, 		255);
		protocoles.put(Agiledomdeg_ID, 		256);
		protocoles.put(Agiledomdeg_Deg, 	257);
		protocoles.put(Agiledomdeg_MaxSucc, 258);
		
		protocoles.put(Agiledomwdeg, 		261);
		protocoles.put(Agiledomwdeg_ID, 	262);
		protocoles.put(Agiledomwdeg_Deg, 	263);
		protocoles.put(Agiledomwdeg_MaxSucc,264);
		/* ===================================================================================================== */
		protocoles.put(ZABTDORetro, 		321);
		protocoles.put(ZABTDORetroMinDom,	322);
		protocoles.put(ZABTDORetroA2ndL, 	323);
		protocoles.put(ZABTDORetroDomPS, 	324);
		protocoles.put(ZABTDORetroDomPAll,	325);
		protocoles.put(ZABTDORetroHighest, 	326);
		/* ===================================================================================================== */

		protocoles.put(Adopt, 1001);
		protocoles.put(BnBAdopt, 1002);
		protocoles.put(AFB, 1003);

		protocoles.put(ABTcf, 2000);

		protocoles.put(DBA, 3000);

	}

	public static AbstractMaster getMaster(String protocol) {
		if (protocoles.keySet().contains(protocol)) {
			switch (protocoles.get(protocol)) {

				/* ===================================================================================================== */
				case 0 : return new SBTMaster();
				/* ===================================================================================================== */
				case 1 : return new ABTMaster();
				case 2 : return new ABTHybMaster();
				case 3 : return new ABTacMaster();
				case 4 : return new ABTdacMaster();
				/* ===================================================================================================== */
				case 101 : return new AFCMaster(false);
				case 102 : return new AFCMinDomMaster();
				case 103 : return new AFCDomDegMaster();
				case 104 : return new AFCTriggeredNogoodMaster();
				case 105 : return new AFCEstimatedMaster(AFCEstimatedMaster.MinDom);
				case 106 : return new AFCEstimatedMaster(AFCEstimatedMaster.DomDeg);
				//--------------------
				case 108 : return new AFCDBJMaster();
				case 109 : return new DDBJMaster();
				/* ===================================================================================================== */
				case 111 : return null;
				case 112 : return null;
				case 113 : return null;
				case 114 : return null;
				/* ===================================================================================================== */
				case 121 : return new AFCMaster(true);
				/* ===================================================================================================== */
				case 132 : return null;
				case 133 : return null;
				case 134 : return null;
				case 135 : return null;
				/* ===================================================================================================== */
//				case 140 : return new AFCngMaster2();
				case 141 : return new AFCngMaster();
				case 142 : return new AFCngMinDomMaster();
				case 143 : return new AFCngDomDegMaster();
				case 144 : return new AFCngTriggeredNogoodMaster();
				case 145 : return new AFCngApproximateMinDomMaster(AFCngApproximateMinDomMaster.MinDom);
				case 146 : return new AFCngApproximateMinDomMaster(AFCngApproximateMinDomMaster.MinDomDeg);
				/* ===================================================================================================== */
				case 151 : return null;
				case 152 : return null;
				case 153 : return null;
				case 154 : return null;
				/* ===================================================================================================== */
				case 161 : return new AFCTreeMaster();
				/* ===================================================================================================== */
				case 162 : return null;
				case 163 : return null;
				case 164 : return null;
				case 165 : return null;
				/* ===================================================================================================== */
				/* ===================================================================================================== */

				/* ===================================================================================================== */
				case 211 : return new ABTDOMaster(ABTDOMaster.Nogood_triggered);
				case 212 : return new ABTDOMaster(ABTDOMaster.MinDom);
				case 213 : return new ABTDOMaster(ABTDOMaster.Random);
				case 214 : return new ABTDOMaster(ABTDOMaster.MinDom_Single);
				case 215 : return new ABTDOMaster(ABTDOMaster.Random_Single);
				/* ===================================================================================================== */
				case 221 : return new ABTDORetroMaster(ABTDORetroMaster.MinDomain);
				case 222 : return new ABTDORetroMaster(ABTDORetroMaster.MinDomain);
				case 223 : return new ABTDORetroMaster(ABTDORetroMaster.After_Second_Last);
				case 224 : return new ABTDORetroMaster(ABTDORetroMaster.Dom_Pos_Single);
				case 225 : return new ABTDORetroMaster(ABTDORetroMaster.Dom_Pos_All);
				case 226 : return new ABTDORetroMaster(ABTDORetroMaster.Highest);
				/* ===================================================================================================== */
//				case 231 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom);
//				case 232 : return new AgileABTMaster(AgileABTMaster.All);
//				case 233 : return new AgileABTMaster(AgileABTMaster.OrderOnly);
//				case 234 : return new AgileABTMaster(AgileABTMaster.Neighbors);
//				case 235 : return new AgileABTMaster(AgileABTMaster.OrderNeigh);
//				case 236 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_ID);
//				case 237 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_Deg);
//				case 238 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.MinDom, AgileABTMaster.Topological_MaxSucc);
//				case 241 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg);
//				case 242 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_ID);
//				case 243 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_Deg);
//				case 244 : return new AgileABTMaster(AgileABTMaster.OrderOnly, AgileABTMaster.DomDeg, AgileABTMaster.Topological_MaxSucc);
//				/* ===================================================================================================== */
//				/* ===================================================================================================== */
//				case 251 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom);
//				case 252 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_ID);
//				case 253 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_Deg);
//				case 254 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.MinDom, AgileMaster.Topological_MaxSucc);
//				case 255 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg);
//				case 256 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_ID);
//				case 257 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_Deg);
//				case 258 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomDeg, AgileMaster.Topological_MaxSucc);
//				case 261 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg);
//				case 262 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_ID);
//				case 263 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_Deg);
//				case 264 : return new AgileMaster(AgileMaster.OrderOnly, AgileMaster.DomWDeg, AgileMaster.Topological_MaxSucc);
				/* ===================================================================================================== */
				/* ===================================================================================================== */
				case 321 : return new ZABTDORetroMaster(ZABTDORetroMaster.MinDomain);
				case 322 : return new ZABTDORetroMaster(ZABTDORetroMaster.MinDomain);
				case 323 : return new ZABTDORetroMaster(ZABTDORetroMaster.After_Second_Last);
				case 324 : return new ZABTDORetroMaster(ZABTDORetroMaster.Dom_Pos_Single);
				case 325 : return new ZABTDORetroMaster(ZABTDORetroMaster.Dom_Pos_All);
				case 326 : return new ZABTDORetroMaster(ZABTDORetroMaster.Highest);
				/* ===================================================================================================== */

				case 3000 : return new DBAmaster();
				default : throw new Error("Unknown protocol: " + protocol);
			}
		}
		throw new Error("Unknown protocol: " + protocol);
	}

	public static SortedMap<String, Integer> getProtocols(String type) {
		if (type.equals(SDisCSP)) { return protocoles; }
		if (type.equals(SDCOP)) { return protocoles; }
		if (type.equals(CDisCSP)) { return protocoles; }
		if (type.equals(CDCOP)) { return protocoles; }
		throw new Error("Invalid protocol type : " + type);
	}

	public static String[] getProtocolsName(String type) {
		SortedMap<String, Integer> protocols = getProtocols(type);
		Iterator<String> it = protocols.keySet().iterator();
		int i = 0;
		String[] ans = new String[protocols.size()];
		while (it.hasNext()) {
			ans[i++] = it.next();
		}
		return ans;
	}
}
