/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * dischoco
 * SimulationOnBenchmarks.java
 */ 
package gui.simulation;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingWorker;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.CategoryLabelPositions;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.DefaultCategoryDataset;

import dischoco.Protocols;

import dischoco.DisChoco;
import benchmark.random.RBDisCSPGenerator;
import gui.visualizing.Edge;
import gui.visualizing.Graph;
import gui.visualizing.GraphPanel;
import gui.visualizing.TreeDrawing;
import gui.visualizing.Vertex;
import kernel.order.AgentComplexity;
import kernel.order.AgentOrdering;
import kernel.order.LexicographicAO;
import kernel.order.MaxDegPlusMinDomAO;
import kernel.order.MinDomDegAO;
import kernel.order.RandomAO;
import kernel.order.TreeAO;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import util.Commands;
import util.IntArray;
import util.Tools;
import util.graph.PriorityTree;
import util.parser.InstanceParser;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */

@SuppressWarnings("serial")
public abstract class SimulationOnBenchmarks extends JPanel {

	public static final short AgentsLoad = 1;
	public static final short MessagesVisualizing = 2;
	protected short typeOfVisualization = AgentsLoad;
	public static final String TreeAO_PANEL = "TreeAO_Panel";
	public static final String GraphAO_PANEL = "GraphAO_Panel";
	public static final String AgentsLoad_PANEL = "AgentsLoad_Panel";

	public final static int EAST_PANEL_WIDTH = 360;
	
	protected boolean isRunning;
	protected InstanceParser parser;
	protected DisProblem disProb;
	protected static final String instructions =
		"<html>" + 
		"<b><h2><center> Instructions for Annotations </center></h2></b>" + 
		"<pre>" + 
		"To do." +
		"</pre>";
	protected ChartPanel chartPanel = null;
	protected JFreeChart chart = null;
	protected DefaultCategoryDataset dataset = null;
	protected IntArray order = null;
	protected File problem = null;
	protected HashMap<Integer, AgentComplexity> complexities = null;
	protected Graph<Vertex, Edge> graph = null;
	protected GraphPanel graphPanel = null;
	protected TreeDrawing treePanel = null;
	protected Dimension size = null;
	protected PriorityTree tree = null;
	protected JButton helpButton = null;
	protected JComboBox agentsOrdering = null;
	protected JComboBox algorithmsBox = null;
	protected JComboBox communicationBox = null;
	protected JDialog helpDialog = null;
	protected JButton runButton = null;
	protected JButton pauseButton = null;
	protected JButton stopButton = null;
	protected DisSolverSimulator solver = null;
	protected JPanel controlPanel = null;
	protected JPanel eastPanel = null;
	protected JPanel westPanel = null;
	protected JPanel propertiesPanel = null;
	protected JPanel plotPanel = null;
	protected JPanel simulatorPanel = null;
	protected JPanel algorithmPanel = null;
	protected JPanel communicationPanel = null;
	protected JPanel helpControls = null;
	
	/**
	 * 
	 */
	public SimulationOnBenchmarks(File file, Dimension d, short visualization) {
		super();
		this.typeOfVisualization = visualization;
		this.dataset = new DefaultCategoryDataset();
		setSize(d);
		setPreferredSize(d);
		size = d;
		problem = file;
		String fileName = this.problem.getAbsolutePath();
		parser = new InstanceParser(fileName);
		parser.loadInstance(fileName);
		parser.parse(false);
		if(this.parser.isComplexModel()){ //TODO 
			this.parser.convertToSimpleModel();
		}
		complexities = parser.getAgentsComplexity();
		graph = parser.getGraph(new Dimension(650, 435));
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		if (fileName.endsWith("xml.bz2")) {
			try {
				Process p = Runtime.getRuntime().exec("bunzip2 -c " + fileName);
				document = sxb.build(p.getInputStream());
				p.waitFor();
				p.exitValue();
				p.destroy();
			} catch (Exception e) {
				Container c = getParent();
				while( !(c instanceof DisChoco) ){
					c = c.getParent();
				}
				((DisChoco) c).println("Problem with " + fileName);
				e.printStackTrace();
			}
		}
		try { document = sxb.build(file);
		} catch (Exception e) {}
		Element gui = document.getRootElement().getChild(RBDisCSPGenerator.GuiPresentation);
		setPresentation(gui);
		setLayout(new BorderLayout());
		add(getWestPanel(), BorderLayout.WEST);
		add(getEastPanel(), BorderLayout.CENTER);
	}

	protected abstract void setPresentation(Element guiPesentation);
	protected abstract JPanel getPropertiesPanel();
	
	/**
	 * 
	 * @return
	 */
	public JButton getRunButton() {//filePath
		if (runButton == null) {
			runButton = new JButton(new ImageIcon(getClass().getResource(DisChoco.IconsPath + "play.png")));
			runButton.setEnabled(true);
			runButton.setToolTipText("Start Experiment Run execution");
			runButton.addActionListener(new ActionListener() {
				@SuppressWarnings("unchecked")
				public void actionPerformed(ActionEvent e) {
					runButton.setEnabled(false);
					pauseButton.setEnabled(true);
					stopButton.setEnabled(true);
					SwingWorker worker = new SwingWorker() {
						protected Object doInBackground() throws Exception {
							startRun();
							return null;
						}
					};
					worker.execute();
				}
			});
		}
		return runButton;
	}

	/**
	 * 
	 */
	protected void startRun() {
		if(problem == null){
			JOptionPane.showMessageDialog(this, 
					"Please load the problem file", 
					"No file experiments loaded", 
					JOptionPane.ERROR_MESSAGE);
			stop();
			return;
		}
		Container c = getParent();
		while(! (c instanceof DisChoco)){
			c = c.getParent();
		}
//		Logger.LOG_DEBUG = true;
		String protocol = (String) getAlgorithmsBox().getSelectedItem();
		String orderings = (String) getAgentsOrdering().getSelectedItem();
		String communicationType = (String) getCommunicationBox().getSelectedItem();
		disProb = new DisProblem(parser, protocol);
		solver = new DisSolverSimulator(disProb);
		solver.setAgentsLoad(this.dataset);
		solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings));
		if(communicationType.equals(Commands.SLOW_COM)){
			solver.setSlowCommunication();
		} else if(communicationType.equals(Commands.MEDIUM_COM)){
			solver.setSlowCommunication();
		}
		solver.addCommunicationMetric();
		solver.addNCCCMetric();
		if(isRunning){
			((DisChoco) c).println("Resume ...");
			solver.resumeThreads();
			stop();
			((DisChoco) c).println("Congratulation!!");  
		} else {
			this.isRunning = true;
			((DisChoco) c).println("Starting solving the problem...");
			((DisChoco) c).println("problem: " + problem);
			solver.solve();
			String s = "================================================";
			((DisChoco) c).printlnWarning("\nprotocol: " + protocol + "\n" + s + s + s + s);
			((DisChoco) c).printlnWarning(solver.getGlobalSolution());
			HashMap<String, Double> runStatistics = solver.getStatistics();
			ArrayList<String> keys = disProb.master.getStatisticsKeys();
			((DisChoco) c).printlnWarning(Tools.toString(keys) + "\n" + Tools.statisticsToString(keys, runStatistics));
			((DisChoco) c).printlnWarning(s + s + s + s + "\n");
			stop();
			((DisChoco) c).printlnWarning("Congratulation!!");  
		}
	}

	/**
	 * 
	 */
	protected void stop() {
		runButton.setEnabled(true);
		pauseButton.setEnabled(false);
		stopButton.setEnabled(false);
		isRunning = false;
		solver.stoptRunning();
	}
	/**
	 * 
	 * @return
	 */
	public JButton getPauseButton() {
		if (pauseButton == null) {
			pauseButton = new JButton(new ImageIcon(getClass().getResource(DisChoco.IconsPath + "pause.png")));
			pauseButton.setEnabled(false);
			pauseButton.setToolTipText("Pause execution after finishing current problem");
			pauseButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (isRunning) {
						runButton.setEnabled(true);
						pauseButton.setEnabled(false);
						System.out.println("Pause solving the problem...");
						solver.suspendThreads();
					}
				}
			});
		}
		return pauseButton;
	}

	/**
	 * 
	 * @return
	 */
	public JButton getStopButton() {
		if (stopButton == null) {
			stopButton = new JButton(new ImageIcon(getClass().getResource(DisChoco.IconsPath + "stop.png")));
			stopButton.setEnabled(false);
			stopButton.setToolTipText("Stop Experiment Run execution");
			stopButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (isRunning) {
						stop();
//						println("Stop solving the problem");
					}
				}
			});
		}
		return stopButton;
	}

	/**
	 * This method initializes WestPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getWestPanel() {
		if(westPanel == null){
			westPanel = new JPanel(new BorderLayout());
			westPanel.add(getPropertiesPanel(), BorderLayout.CENTER);
			westPanel.setBorder(BorderFactory.createEtchedBorder());
		}
		return westPanel;
	}

	public JComboBox getAgentsOrdering() {
		if (agentsOrdering == null) {
			String[] items = {Commands.LexicoAO, Commands.RandomAO, Commands.MinDomDegAO, Commands.MaxDegPlusMinDomAO, Commands.TreeAO};
			agentsOrdering = new JComboBox(items);
			agentsOrdering.setBounds(new Rectangle(100, 50, 150, 22));
			agentsOrdering.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						chart = null;
						plotEastPanel((String) e.getItem());
					}
				}
			});
			if(this.typeOfVisualization == SimulationOnBenchmarks.MessagesVisualizing){
				plotGraphAO();
			}
			System.out.println("Agent ordering : " + getOrder());
		}
		return agentsOrdering;
	}

	/**
	 * This method initializes figurePanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getEastPanel() {
		if (eastPanel == null) {
			eastPanel = new JPanel(new BorderLayout());
			Dimension d = new Dimension(size.width, size.height);
			eastPanel.setSize(d);
			eastPanel.setPreferredSize(d);
			eastPanel.add(getSimulatorPanel(d.width), BorderLayout.NORTH);
			eastPanel.add(getPlotPanel(), BorderLayout.CENTER);
		}
		return eastPanel;
	}


	/**
	 * This method initializes plotsPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected JPanel getPlotPanel() {
		if (plotPanel == null) {
			plotPanel = new JPanel(new CardLayout());
			Dimension d = new Dimension(size.width-EAST_PANEL_WIDTH, size.height-GraphPanel.NORTH_PANEL_HEIGHT);
			plotPanel.setSize(d);
			plotPanel.setPreferredSize(d);
			plotPanel.setBorder(BorderFactory.createEtchedBorder());
			if(this.typeOfVisualization == SimulationOnBenchmarks.MessagesVisualizing){
				getWestPanel().add(getGraphPanel().getLayoutControls(), BorderLayout.SOUTH);
			} else if(this.typeOfVisualization == SimulationOnBenchmarks.AgentsLoad) {
				plotPanel.add(getChartPanel(), AgentsLoad_PANEL);
				CardLayout cards = (CardLayout) (plotPanel.getLayout());
				cards.show(plotPanel, AgentsLoad_PANEL);
			}
		}
		return plotPanel;
	}


	/**
	 * @param keySet
	 */
	protected void setKeys() {
		this.dataset = new DefaultCategoryDataset();
		if(this.parser.isSimpleModel()){
			for(int i=0 ; i<this.order.size() ; i++){
				int agId = this.order.get(i);
				String name = agId+"";//+":"+this.parser.getAgentName(agId);
				dataset.addValue(0D, name, name);
			}
		} else if(this.parser.isComplexModel()){  
			for(int i=0 ; i<this.order.size() ; i++){
				int agId = this.order.get(i);
				String name = agId+"";//+":"+this.parser.getAgentName(agId);
				dataset.addValue(0D, name, name);
			}
		}
	}
	/**
	 * @return
	 */
	protected IntArray getOrder() {
		String selectedItem = (String) getAgentsOrdering().getSelectedItem();
		if (selectedItem.equals(Commands.TreeAO)) {
			order = IntArray.fromString("["+getTree().toDFSString()+"]");
		} else if (selectedItem.equals(Commands.LexicoAO)) {
			order = LexicographicAO.buildpriorityTree(complexities);
		} else if (selectedItem.equals(Commands.RandomAO)) {
			order = RandomAO.buildpriorityTree(complexities);
		} else if (selectedItem.equals(Commands.MinDomDegAO)) {
			order = MinDomDegAO.buildpriorityTree(complexities);
		} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
			order = MaxDegPlusMinDomAO.buildpriorityTree(complexities);
		}
		if(this.typeOfVisualization == SimulationOnBenchmarks.AgentsLoad) {
			setKeys();
		}
		return order;
	}

	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected GraphPanel getGraphPanel() {
		if (graphPanel == null) {
			graphPanel = new GraphPanel(graph, new Dimension(size.width-EAST_PANEL_WIDTH, size.height));
		}
		return graphPanel;
	}


	/**
	 * This method initializes randomAOPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	protected TreeDrawing getTreePanel() {
		if (treePanel == null) {
			treePanel = new TreeDrawing(getTree(), new Dimension(size.width-EAST_PANEL_WIDTH, size.height));
		}
		return treePanel;
	}

	/**
	 * @return
	 */
	protected PriorityTree getTree() {
		if(tree == null){
			tree = TreeAO.buildpriorityTree(this.complexities);
		}
		return tree;
	}

	protected void plotEastPanel(String selectedItem) {
		Container c = getParent();
		while( !(c instanceof DisChoco) ){
			c = c.getParent();
		}
		((DisChoco) c).println("Agent ordering : " + getOrder());
		if(typeOfVisualization == SimulationOnBenchmarks.MessagesVisualizing){
			if (selectedItem.equals(Commands.TreeAO)) {
				plotTreeAO();
			} else if (selectedItem.equals(Commands.LexicoAO)) {
				plotGraphAO();
			} else if (selectedItem.equals(Commands.RandomAO)) {
				plotGraphAO();
			} else if (selectedItem.equals(Commands.MinDomDegAO)) {
				plotGraphAO();
			} else if (selectedItem.equals(Commands.MaxDegPlusMinDomAO)) {
				plotGraphAO();
			}
		} else{
			chooseOrder(selectedItem);
			getBarChart();
		}
		
	}

	protected void chooseOrder(String selectedItem) {
		Container c = getParent();
		while( !(c instanceof DisChoco) ){
			c = c.getParent();
		}
		((DisChoco) c).println("Agent ordering : " + getOrder());
	}

	/**
	 * 
	 */
	protected void plotGraphAO() {
		getEastPanel().add(getPlotPanel(), BorderLayout.CENTER);
		getPlotPanel().add(getGraphPanel().getCanvas(new Dimension(size.width-EAST_PANEL_WIDTH, size.height)), GraphAO_PANEL);
		getPlotPanel().setBorder(BorderFactory.createEtchedBorder());
		CardLayout cards = (CardLayout) (getPlotPanel().getLayout());
		cards.show(getPlotPanel(), GraphAO_PANEL);
	}

	protected void plotTreeAO() {
		Container c = getParent();
		while( !(c instanceof DisChoco) ){
			c = c.getParent();
		}
		getEastPanel().add(getPlotPanel(), BorderLayout.CENTER);
		((DisChoco) c).println("Agent ordering (BFS) : " + getOrder());
		getTree().position(0);
		getPlotPanel().add(getTreePanel().getCanvas(new Dimension(size.width-EAST_PANEL_WIDTH, size.height)), TreeAO_PANEL);
		CardLayout cards = (CardLayout) (getPlotPanel().getLayout());
		cards.show(getPlotPanel(), TreeAO_PANEL);
	}
	
	/**
	 * This method initializes simulatorPanel
	 * 
	 * @param dimension
	 * @return javax.swing.JPanel
	 */
	public JPanel getSimulatorPanel(int width) {
		if (simulatorPanel == null) {
			simulatorPanel = new JPanel();
			simulatorPanel.setLayout(new FlowLayout());  
			simulatorPanel.setPreferredSize(new Dimension(width, GraphPanel.NORTH_PANEL_HEIGHT));
			simulatorPanel.setMinimumSize(new Dimension(width, GraphPanel.NORTH_PANEL_HEIGHT));
			simulatorPanel.add(getControlPanel());
			simulatorPanel.add(getAlgorithmPanel());
			simulatorPanel.add(getCommunicationPanel());
//			simulatorPanel.add(getHelpControls());
//			simulatorPanel.setBorder(BorderFactory.createEtchedBorder());
		}
		return simulatorPanel;
	}

	/**
	 * @return JPanel
	 */
	public JPanel getControlPanel() { // control 
		if(controlPanel == null){
			controlPanel = new JPanel();
			controlPanel.setBorder(BorderFactory.createTitledBorder(Commands.Control));
			controlPanel.add(getRunButton());	
			controlPanel.add(getPauseButton());	
			controlPanel.add(getStopButton());	
		}
		return controlPanel;
	}

	/**
	 * @return JPanel
	 */
	public JPanel getAlgorithmPanel() {
		if(algorithmPanel == null){
			algorithmPanel = new JPanel();
			algorithmPanel.setBorder(BorderFactory.createTitledBorder(Commands.Algorithm));
			algorithmPanel.add(getAlgorithmsBox());	
		}
		return algorithmPanel;
	}
	
	/**
	 * @return JPanel
	 */
	public JPanel getCommunicationPanel() {
		if(communicationPanel == null){
			communicationPanel = new JPanel();
			communicationPanel.setBorder(BorderFactory.createTitledBorder(Commands.Communication));
			communicationPanel.add(getCommunicationBox());
		}
		return communicationPanel;
	}
	
	/** 
	 * @return JPanel
	 */
	public JPanel getHelpControls() {
		if(helpControls == null){
			helpControls = new JPanel();
			helpControls.setBorder(BorderFactory.createTitledBorder("Help"));
			helpControls.add(getHelpButton());
		}
		return helpControls;
	}
	
	public JButton getHelpButton() {
		if (helpButton == null) {
			helpButton = new JButton("Help");
			helpButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getHelpDialog().pack();
					Container c = getParent();
					while(!(c instanceof JFrame)){
						c = c.getParent();
					}
					JFrame frame = (JFrame) c;
					Point loc = frame.getLocation();
					Dimension size = frame.getSize();
					Dimension d = helpDialog.getSize();
					helpDialog.setLocation(loc.x+(size.width-d.width)/2, loc.y + (size.height-d.height)/2);
					helpDialog.setVisible(true);
				}
			});
		}
		return helpButton;
	}

	protected JDialog getHelpDialog() {
		if (helpDialog == null) {
			helpDialog = new JDialog();
			helpDialog.getContentPane().add(new JLabel(instructions));
		}
		return helpDialog;
	}

	/**
	 * @return Returns the modeBox.
	 */
	public JComboBox getAlgorithmsBox() {
		if (algorithmsBox == null) {
			algorithmsBox = new JComboBox(Protocols.getProtocolsName(Protocols.SDisCSP));
			algorithmsBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent paramItemEvent) {
					
				}
			});
		}
		return algorithmsBox;
	}

	/**
	 * @return Returns the modeBox.
	 */
	public JComboBox getCommunicationBox() {
		if (communicationBox == null) {
			communicationBox = new JComboBox(Commands.communicationTypes);
			communicationBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
				}
			});
		}
		return communicationBox;
	}

	/**
	 * 
	 */
	protected ChartPanel getChartPanel() {
		if (chartPanel == null) {
			chartPanel = new ChartPanel(getBarChart());
			Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
			chartPanel.setPreferredSize(size);
	        chartPanel.setPopupMenu(null);
		}
		return chartPanel;
	}
	
	/**
	 * 
	 * @param dataset
	 */
	protected JFreeChart getBarChart() {
		if(chart == null){
			chart = ChartFactory.createBarChart("", // Title
					"Agents", "Constraint Checks", dataset, PlotOrientation.VERTICAL, true, false, false);
			chart.setBackgroundPaint(Color.white);
			chart.setBorderVisible(true);
			chart.getLegend().setVisible(false);
			CategoryPlot plot = chart.getCategoryPlot();
			plot.setBackgroundPaint(Color.white);
			plot.setRangeGridlinePaint(Color.black);
			plot.setRangeCrosshairVisible(true);
			BarRenderer renderer = (BarRenderer) plot.getRenderer();
			renderer.setDrawBarOutline(false);
			renderer.setItemMargin(-5);
			final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
			rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
			final CategoryAxis domainAxis = plot.getDomainAxis();
			domainAxis.setCategoryLabelPositions(CategoryLabelPositions.UP_90);
			getChartPanel().setChart(chart);
		}
		return chart;
	}
	
}
