package gui.visualizing;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import util.graph.PriorityTree;

public class TreeDrawing extends JPanel {
	
	private static final long serialVersionUID = 897837852917353184L;
	public static final double DEFAULT_ZOOM_PERCENTAGE = 1.08;
	public static final int MAX_ZOOM = 5;
	private RenderingHints hints;
	private PriorityTree tree;
	private JPanel northPanel;
	private JPanel canvas;
	private JButton zoomOut;
	private JButton zoomIn;
	private JButton helpButton;
	private JDialog helpDialog;
	private JComboBox modeBox;
	static final String instructions =
		"<html>" + 
		"<b><h2><center>Instructions for Annotations</center></h2></b>" + 
		"<pre>" + 
		"To do." +
		"</pre>";
	public static final int NORTH_PANEL_HEIGHT = 75;
	private boolean PICKING = true;// transform mode false
	private JPanel zoomControls;
	private JPanel modeControls;
	private JPanel helpControls;
	
	public TreeDrawing(PriorityTree t, Dimension d) {
		super();
		hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		hints.add(new RenderingHints( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY));
		tree = t;
		setSize(d);
		setLayout(new BorderLayout());
		add(getNorthPanel(d), BorderLayout.NORTH);
		add(new JScrollPane(getCanvas(d)), BorderLayout.CENTER);
		setBorder(BorderFactory.createEtchedBorder());
	}
	/**
	 * This method initializes northPanel
	 * 
	 * @param height
	 * @param width
	 * @return javax.swing.JPanel
	 */
	public JPanel getNorthPanel(Dimension d) {
		if (northPanel == null) {
			northPanel = new JPanel();
			northPanel.setLayout(new FlowLayout());
			northPanel.setPreferredSize(new Dimension(d.width, NORTH_PANEL_HEIGHT));
			northPanel.setMinimumSize(new Dimension(d.width, NORTH_PANEL_HEIGHT));
			northPanel.add(getZoomIn());
			northPanel.add(getZoomOut());
			northPanel.add(getZoomControls());
			northPanel.add(getModeControls());
			northPanel.add(getHelpControls());
		}
		return northPanel;
	}

	/**
	 * @return JPanel
	 */
	public JPanel getZoomControls() {
		if(zoomControls == null){
			zoomControls = new JPanel();
			zoomControls.setBorder(BorderFactory.createTitledBorder("Zoom"));
			zoomControls.add(getZoomIn());
			zoomControls.add(getZoomOut());
		}
		return zoomControls;
	}
	/**
	 * @return JPanel
	 */
	public JPanel getModeControls() {
		if(modeControls == null){
			modeControls = new JPanel();
			modeControls.setBorder(BorderFactory.createTitledBorder("Mouse"));
			modeControls.add(getModeComboBox());
		}
		return modeControls;
	}
	/**
	 * @return JPanel
	 */
	public JPanel getHelpControls() {
		if(helpControls == null){
			helpControls = new JPanel();
			helpControls.setBorder(BorderFactory.createTitledBorder("Help"));
			helpControls.add(getHelpButton());
		}
		return helpControls;
	}
	/**
	 * @return Returns the modeBox.
	 */
	public JComboBox getModeComboBox() {
		if (modeBox == null) {
			modeBox = new JComboBox(new String[] {"PICKING", "TRANSFORMING",});
			modeBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent paramItemEvent) {
					if (modeBox.getSelectedItem().equals("PICKING")) {
						PICKING = true;
					} else {
						PICKING = false;
					}
				}
			});
		}
		modeBox.setSelectedItem("PICKING");
		return modeBox;
	}

	public JButton getHelpButton() {
		if (helpButton == null) {
			helpButton = new JButton("Help");
			helpButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getHelpDialog().pack();
					Container c = getParent();
					while(!(c instanceof JFrame)){
						c = c.getParent();
					}
					JFrame frame = (JFrame) c;
					Point loc = frame.getLocation();
					Dimension size = frame.getSize();
					Dimension d = helpDialog.getSize();
					helpDialog.setLocation(loc.x+(size.width-d.width)/2, loc.y + (size.height-d.height)/2);
					helpDialog.setVisible(true);
				}
			});
		}
		return helpButton;
	}

	private JDialog getHelpDialog() {
		if (helpDialog == null) {
			helpDialog = new JDialog();
			helpDialog.getContentPane().add(new JLabel(instructions));
		}
		return helpDialog;
	}
	/**
	 * This method initializes browse
	 * 
	 * @return javax.swing.JButton
	 */
	public JButton getZoomIn() {
		if (zoomIn == null) {
			zoomIn = new JButton();
			zoomIn.setText(" + ");
			zoomIn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					((Canvas) canvas).zoom(1.1f);
				}
			});
		}
		return zoomIn;
	}

	/**
	 * This method initializes browse
	 * 
	 * @return javax.swing.JButton
	 */
	public JButton getZoomOut() {
		if (zoomOut == null) {
			zoomOut = new JButton();
			zoomOut.setText(" - ");
			zoomOut.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					((Canvas) canvas).zoom(1 / 1.1f);
				}
			});
		}
		return zoomOut;
	}
	
	/**
	 * @param d 
	 * @return
	 */
	public JPanel getCanvas(Dimension d) {
		if( canvas == null){
			canvas = new Canvas(d);
		}
		return canvas;
	}
	
	public class Canvas extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener {

		private Vertex pick;
		private boolean pickfixed;
		
		private static final long serialVersionUID = 1L;
		private int xsteps, xoffset, ysteps, yoffset;
		private Dimension preferredSize = null;
		private Dimension minimumSize = null;
		private Dimension maximumSize = null;
		private Collection<Edge> edges = null;
		private HashMap<Integer, Vertex> vertexes = null;  
		private FontMetrics m;
		
		public Canvas(Dimension d){
			super();
			minimumSize = new Dimension(d.width-25, d.height - 100);
			maximumSize = new Dimension((minimumSize.width*MAX_ZOOM), minimumSize.height*MAX_ZOOM);
			preferredSize = minimumSize;
			setBackground(Color.white);
			setSize(preferredSize);
			addMouseWheelListener(this);
			addMouseListener(this);
			makingTree();
		}
		
		public Dimension getPreferredSize() {
			return preferredSize;
		}

		public void mouseClicked(MouseEvent e) { }
		public void mouseEntered(MouseEvent e) { }
		public void mouseExited(MouseEvent e) { }
		public void mousePressed(MouseEvent e) {
			Point p = e.getPoint();
			if(PICKING){
				addMouseMotionListener(this);
				for(int id : vertexes.keySet()){
					Vertex v = vertexes.get(id);
					if(v.contains(p)){
						pick = v;
						break;
					}
				}
				if (pick != null) {
					pickfixed = pick.selected;
					pick.selected = true;
					pick.setCenter(p);
				}
			} else{
				
			}
			repaint();
			e.consume();
		}
		public void mouseReleased(MouseEvent e) {
			if(PICKING){
				removeMouseMotionListener(this);
				if (pick != null) {
					pick.setCenter(e.getPoint());
					pick.selected = pickfixed;
					pick = null;
				}
			} else{
				
			}
			repaint();
			e.consume();
		}
		public void mouseDragged(MouseEvent e) {
			if(PICKING){
				if (pick != null) {
					pick.setCenter(e.getPoint());
				}
			} else {
				
			}
			repaint();
			e.consume();
		}
		public void mouseMoved(MouseEvent e) {}
		
		public void mouseWheelMoved(MouseWheelEvent e) {
			updatePreferredSize(e.getWheelRotation(), e.getPoint());
		}
		
		private void updatePreferredSize(int n, Point p) {
			double d = (double) n;
			d = (n > 0)? (1 / d) : (-d);
			zoom(d);
		}

		private void zoom(double d) {
			int w = (int) ((getWidth()) * d * DEFAULT_ZOOM_PERCENTAGE);
			int h = (int) ((getHeight()) * d * DEFAULT_ZOOM_PERCENTAGE);
			if (w < minimumSize.getWidth()) {
				w = minimumSize.width;
			} else if (w > maximumSize.getWidth()) {
				w = maximumSize.width;
			}
			if (h < minimumSize.getHeight()) {
				h = minimumSize.height;
			} else if (h > maximumSize.getHeight()) {
				h = maximumSize.height;
			}
			preferredSize = new Dimension(w, h);
			makingTree();
			getParent().doLayout();
			revalidate();
			repaint();
		}
		
		public int scale(double f) {
			return (int) (f * xsteps) + xoffset;
		}

		public int scale(int f) {
			return (f * ysteps) + yoffset;
		}
		
		public void paint(Graphics g) {
			Graphics2D gr = ((Graphics2D) g);
	        gr.setColor(Color.white);
			m = gr.getFontMetrics();
	        gr.fillRect(0, 0, preferredSize.width, preferredSize.height);
			gr.setRenderingHints(hints);
	        gr.setStroke(new BasicStroke(1.8f));
			drawTree(g);
		}
		
		private void makingTree(){
			xsteps = (int) (preferredSize.width / (tree.getMaxX() - tree.getMinX() + 1));
			ysteps = (int) (preferredSize.height / (tree.depth() + 1));
			xoffset = (int) (xsteps * (0.5 - tree.getMinX()));
			yoffset = ysteps / 2;
			this.edges = new HashSet<Edge>();
			this.vertexes = new HashMap<Integer, Vertex>();
			makeNodes(tree, 0);
		}
		
		private void drawTree(Graphics g){
			for(Edge e : this.edges){
				e.draw(g);
			}
			for(int id : vertexes.keySet()){
				Vertex v = vertexes.get(id);
				v.draw(g, m);
			}
		}
		
		public void makeNodes(PriorityTree t, int o) {
			for (PriorityTree pp : t.getChildren()) {
				makeLine(t, pp, o);
				makeNodes(pp, o+1);
			}
			makeNode(t, o);
		}

		private void makeLine(PriorityTree p1, PriorityTree p2, int o) {
			Vertex v1 = new Vertex(p1.getRootId(), scale(p1.getPosition()), scale(o)+Vertex.height);
			Vertex v2 = new Vertex(p2.getRootId(), scale(p2.getPosition()), scale(o + 1)+Vertex.height);
			if(!vertexes.keySet().contains(v1.id)){
				vertexes.put(v1.id, v1);
			} else {
				v1 = vertexes.get(v1.id);
			}
			if(!vertexes.keySet().contains(v2.id)){
				vertexes.put(v2.id, v2);
			} else {
				v2 = vertexes.get(v2.id);
			}
			edges.add(new Edge(v1, v2));
		}

		public void makeNode(PriorityTree p, int order) {
			int id = p.getRootId();
			int x = scale(p.getPosition());
			int y = scale(order)+Vertex.height;
			Vertex v = new Vertex(id, x, y);
			if(!vertexes.keySet().contains(v.id)){
				vertexes.put(id, v);
			}
		}
	}
	
}
