/** 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
 * 
 * @version 2.0
 * DisChoco
 * Vertex.java
 */ 
package gui.visualizing;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

/**
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 * 
 */
public class Vertex {
	private static final long serialVersionUID = 2020459878177381947L;
	String label = "";
	int id, rad, len;
	public final static int width = 32, height = 32;
	Ellipse2D.Double shape = null;
	double x, y, dx, dy;
	public boolean selected = false;
	private double scale = 1.0;
	public Vertex(int id, double x, double y) {
		super();
		this.x = x;
		this.y = y;
		this.dx = 0;
		this.dy = 0;
		this.id = id;
		label = "A" + id;
//		shape = new Ellipse2D.Double(x - width*scale / 2, y - height*scale / 2, width*scale, height*scale);
	}
	/**
	 * @param point
	 */
	public void setCenter(Point p) {
		x = p.getX();
		y = p.getY();
//		shape = new Ellipse2D.Double(x - width/2, y - height/2, width, height);
	}
	
	public void draw(Graphics g, FontMetrics m) {
		Graphics2D gr = (Graphics2D) g;
		if (selected) gr.setColor(Color.yellow);
		else gr.setColor(Color.red);
		shape = new Ellipse2D.Double(x - width/2, y - height/2, width, height);
		gr.fill(shape);
		if (selected) gr.setColor(Color.blue);
		else gr.setColor(Color.white);
		rad = m.getAscent();
		len = m.stringWidth(label);
		int x = (int) shape.getCenterX();
		int y = (int) shape.getCenterY();
		gr.drawString(label, (int) (x - len / 2), (int) (y + (rad - m.getDescent()) / 2));
		if (selected) gr.setColor(Color.blue);
		else gr.setColor(Color.black);
		gr.draw(shape);
	}
	
	public boolean contains(Point p) {
		return shape.contains(p);
	}
	public Point2D getCenter() {
		return new Point2D.Double(x, y);
	}
	public double getX() {
		return x;
	}
	public double getY() {
		return y;
	}
	/**
	 * @param scale
	 */
	public void scale(double scale) {
		if(this.scale != scale){
			this.scale = scale;
			shape = new Ellipse2D.Double(x - width*scale/2, y - height*scale/2, width*scale, height*scale);
		}
	}
}
