package kernel.constraints.integer;

public interface BinRelation {

	/**
	 * return true if couple (x,y) is feasible according to the definition of the relation. e.g if the relation is defined with infeasible tuples, it
	 * returns true if (x,y) is one of them.
	 * 
	 * @param x
	 * @param y
	 * 
	 */
	public boolean checkCouple(int x, int y);

	/**
	 * Test whether the couple (x,y) is consistent
	 * 
	 * @param x
	 * @param y
	 * @return true if (x,y) is a consistent couple
	 */
	public boolean isConsistent(int x, int y);
	
	/**
	 * return the number of supports for value x
	 * @param x
	 * @return
	 */
	public int getNumberOfSupports(int x);
	
	/**
	 * return the number of conflicts for value x
	 * @param x
	 * @return
	 */
	public int getNumberOfConflicts(int x);

}
