package kernel.order;

import java.util.HashMap;
import java.util.Iterator;

import util.IntArray;
import util.Tools;

/**
 * A Class for Min-Domain Degree ordering
 */
public class MinDomDegAO extends CentralizedAO implements ChainOrder {

	public MinDomDegAO() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see order.CentralizedAgentOrdering#buildAgentsOrder()
	 */
	public IntArray buildAgentsOrder() {
		return this.buildAgentsOrderForComplex();
//		IntArray order = new IntArray();
//		int n = this.agentsComplexity.keySet().size();
//		double[] dom_deg = new double[n];
//		int i = 0;
//		Iterator<Integer> it = this.agentsComplexity.keySet().iterator();
//		while (it.hasNext()) {
//			int agId = it.next();
//			// agComp= domSize+","+nbConstraints+","+"nbVariables;
//			AgentComplexity agComp = this.agentsComplexity.get(agId);
//			if (agComp.getNumberConstraints() == 0)
//				dom_deg[i++] = Double.POSITIVE_INFINITY;
//			else
//				dom_deg[i++] = ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints();
//		}
//		Object[] agIds = this.agentsComplexity.keySet().toArray();
//		for (i = 0 ; i < this.agentsComplexity.size() ; i++) {
//			int idx = util.Tools.indexOfMin(dom_deg);
//			dom_deg[idx] = Double.POSITIVE_INFINITY;
//			order.add((Integer) agIds[idx]);
//		}
//		return order;
	}
	
	/**
	 * 
	 */
	public IntArray buildAgentsOrderForComplex() {
		IntArray order = new IntArray();
		int n = this.agentsComplexity.keySet().size();
		HashMap<Integer, Double> dom_deg = new HashMap<Integer, Double>();
		Iterator<Integer> it = this.agentsComplexity.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			AgentComplexity agComp = this.agentsComplexity.get(agId);
//			System.out.println(agId+" : "+agComp.getLocalAgents());
			if (agComp.getNumberConstraints() == 0){
				dom_deg.put(agId, Double.POSITIVE_INFINITY);
			} else{
				dom_deg.put(agId, ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints());
			}
		}
		while (order.size()<n) {
			int agId = Tools.MinOf(dom_deg);
			order.add(agId);
			dom_deg.remove(agId);
			IntArray localAgents = this.agentsComplexity.get(agId).getLocalAgents();
			IntArray local_agents = localAgents.minus(order);
			while(!order.contains(localAgents)){
				HashMap<Integer, Double> local_dom_deg = new HashMap<Integer, Double>();
				for(int id : local_agents){
					local_dom_deg.put(id, dom_deg.get(id));
				}
				int localId = Tools.MinOf(local_dom_deg);
				order.add(localId);
				dom_deg.remove(localId);
				local_agents.remove(localId);
			}
		}
//		System.out.println(order);
//		System.exit(0);
		return order;
	}

	public static IntArray buildpriorityTree(HashMap<Integer, AgentComplexity> complexities) {
		IntArray order = new IntArray();
		int n = complexities.keySet().size();
		double[] dom_deg = new double[n];
		int i = 0;
		Iterator<Integer> it = complexities.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			// agComp= domSize+","+nbConstraints+","+"nbVariables;
			AgentComplexity agComp = complexities.get(agId);
			if (agComp.getNumberConstraints() == 0)
				dom_deg[i++] = Double.POSITIVE_INFINITY;
			else
				dom_deg[i++] = ((double) agComp.getMaxDomain()) / agComp.getNumberConstraints();
		}
		Object[] agIds = complexities.keySet().toArray();
		for (i = 0 ; i < complexities.size() ; i++) {
			int idx = util.Tools.indexOfMin(dom_deg);
			dom_deg[idx] = Double.POSITIVE_INFINITY;
			order.add((Integer) agIds[idx]);
		}
		return order;
	}

}
