package simple.dcsp.abtfamily.abt;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.TerminateMsgHandler;
import kernel.events.AgentEvent;
import simple.SimpleAgent;
import simple.dcsp.abtfamily.IdleEvent;
import util.BooleanLock;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.Logger;
import util.VarAssignment;

/**
 * termination detection (Silaghi et al. 2000g)
 * 
 * @author redouane
 * 
 */
public class ABTTermDetection extends TerminateMsgHandler {

	protected boolean myAgentIsInitialEnd = false;
	protected HashMap<Integer, Instantiation> acceptedMsg;
	public BooleanLock whenIdleLock = new BooleanLock(false);
	protected Instantiation prevSentAccepted = null;

	public ABTTermDetection(AbstractAgent agent) {
		super(agent);
		this.acceptedMsg = new HashMap<Integer, Instantiation>();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#processTerminationDetection(.com.Message)
	 */
	public boolean processTerminationDetection(Message msg) throws StopException {
		Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
		if (accepted != null) {
			int senderId = msg.getSenderId();
			if (!this.acceptedMsg.containsKey(senderId)) {
				this.acceptedMsg.put(senderId, accepted);
			} else {
				Instantiation stored = this.acceptedMsg.get(senderId);
				if (accepted.compareTimeCounters(stored) > 0) {
					this.acceptedMsg.put(senderId, accepted);
				}
			}
			this.myAgent.log("***********processTerminationDetection:\n"+
					this.acceptedMsg+""+accepted, Logger.INFO_LEVEL);
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.TerminateMsgHandler#handleAgentEvent(events.AgentEvent )
	 */
	@Override
	public void handleAgentEvent(AgentEvent agentEvent) {
		if (agentEvent instanceof IdleEvent && !this.myAgent.isFinished()) {
			this.tryToSendAccepted();
		}
		if (agentEvent instanceof kernel.events.AgentFinishedEvent) {
			this.askMasterToTerminate(true);
		}
	}

	/**
	 * 
	 */
	public void tryToSendAccepted() {
		if (this.allAcceptedMsgHaveBeenReceived()) {
			Instantiation accepted = this.BuildAcceptedIfAllCompatible();
			if (accepted != null) {
				if (this.prevSentAccepted != null && this.prevSentAccepted.isSubSetOf(accepted)) { return; }
				this.prevSentAccepted = accepted;
				IntArray higbNeighbors = this.myAgent.communicator.getHigherNeighbors0();
				for (int i = 0 ; i < higbNeighbors.size() ; i++)
					this.sendAccepted(accepted, higbNeighbors.get(i));
				if (this.isMyAgentInitialEnd()) {
					this.sendAccepted(accepted, AbstractMaster.MasterId);
				}
			}
		}

	}

	protected boolean isMyAgentInitialEnd() {
		return this.myAgentIsInitialEnd;
	}

	protected Instantiation BuildAcceptedIfAllCompatible() {
		Instantiation union = this.myAgent.agentView.getContextsOfHigherAgents();
		VarAssignment mySol = ((SimpleAgent) this.myAgent).var.getVarAssignment();
		union.add(mySol);
		this.myAgent.log("mySol:"+mySol, Logger.INFO_LEVEL);
		Iterator<Integer> it = this.acceptedMsg.keySet().iterator();
		while (it.hasNext()) {
			Instantiation ass = this.acceptedMsg.get(it.next());
			if (union.compatible(ass)) {
				union.priorityMerge(ass);
			} else {
				this.myAgent.log("not compatible:"+union+"\nwith"+ass, Logger.INFO_LEVEL);
				return null;
			}
		}
		return union;
	}

	protected void sendAccepted(Instantiation accepted, int receiverID) {
		Message msg = null;
		msg = this.createMessage();
		msg.addContents(Commands.ACCEPTED, accepted.clone());
		msg.setReceiverId(receiverID);
		this.myAgent.sendMessage(msg);
	}

	protected synchronized boolean allAcceptedMsgHaveBeenReceived() {
		IntArray lowerNeighbors = this.myAgent.communicator.getLowerNeighbors0();
		if (this.acceptedMsg.size() == lowerNeighbors.size()) {
			Iterator<Integer> it = acceptedMsg.keySet().iterator();
			while (it.hasNext()) {
				Integer key = it.next();
				Instantiation ass = acceptedMsg.get(key);
				if (ass.isEmpty()) {
					this.myAgent.log(key+" ass:"+ass, Logger.INFO_LEVEL);
					return false;
				}
			}
			return true;
		}
		this.myAgent.log("************* acceptedSize: "+this.acceptedMsg.size()+" <> "+
				"************* lower Neighbors: "+lowerNeighbors.size(), Logger.INFO_LEVEL);
		return false;
	}

	public void sendInitialEndAgentMessage() {
		IntArray highNeighbors = this.myAgent.communicator.getHigherNeighbors0();
		this.myAgentIsInitialEnd = (highNeighbors.size() == 0);
		Message msg = this.createMessage();
		msg.addContents(Commands.INITIAL_END_AGENT, new Boolean(this.myAgentIsInitialEnd));
		msg.setReceiverId(AbstractMaster.MasterId);
		this.myAgent.sendMessage(msg);
		IntArray lowerNeighbors = this.myAgent.communicator.getLowerNeighbors0();
		for (int neig : lowerNeighbors) {
			this.acceptedMsg.put(neig, new Instantiation(this.myAgent.agentOrder));
		}
	}

}
