package simple.dcsp.abtfamily.abtdo;

import java.util.HashMap;

import kernel.MasterObserver;
import kernel.com.Message;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.Logger;

/**
 * termination detection (Silaghi et al. 2000)
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class TerminationDetector extends MasterObserver {

	protected HashMap<Integer, Instantiation> acceptedMsg;
	protected HashMap<Integer, CurrentOrder> currentOrders = null;
	protected IntArray InitEndAgents = null;
	protected CurrentOrder bestOrder = null;

	public TerminationDetector() {
		super();
		this.acceptedMsg = new HashMap<Integer, Instantiation>();
		this.currentOrders = new HashMap<Integer, CurrentOrder>();
		this.InitEndAgents = new IntArray();
	}

	@Override
	public boolean handleReceivedMessage(Message msg) {
		if (msg.getType().equals(Commands.ORDER)) {
			this.ProcessOrder(msg);
			this.checkEnd();
		}
		if (msg.getType().equals(Commands.TERMINATION_DETETION)) {
			Instantiation accepted = (Instantiation) msg.get(Commands.ACCEPTED);
			CurrentOrder rcvOrder = (CurrentOrder) msg.get(Commands.ORDER);
			int senderId = msg.getSenderId();
			this.setBestOrder(rcvOrder);
			int cmp = this.bestOrder.compare(rcvOrder);
			if (cmp <= 0) {
				this.currentOrders.put(senderId, rcvOrder);
				Instantiation stored = this.acceptedMsg.get(senderId);
				if (accepted.compareTimeCounters(stored) > 0) {
					this.acceptedMsg.put(senderId, accepted);
				}
			} else {
				if (cmp > this.bestOrder.getOrder().indexOf(senderId) + 1) {
					if (this.currentOrders.containsKey(senderId)) {
						CurrentOrder senderOrder = this.currentOrders.get(senderId);
						if (senderOrder.compare(rcvOrder) < 0) {
							this.currentOrders.put(senderId, rcvOrder);
						}
						Instantiation stored = this.acceptedMsg.get(senderId);
						if (accepted.compareTimeCounters(stored) > 0) {
							this.acceptedMsg.put(senderId, accepted);
						}
					} else {
						this.currentOrders.put(senderId, rcvOrder);
						this.acceptedMsg.put(senderId, accepted);
					}
				}
			}
			this.checkEnd();
		}
		return true;
	}

	private void ProcessOrder(Message msg) {
		this.setBestOrder((CurrentOrder) msg.get(Commands.ORDER));
	}

	private IntArray computeInitialEndAgents() {
		this.InitEndAgents = new IntArray();
		for (int agId : this.bestOrder.getOrder()) {
			if (this.GammaPlus(agId).size() == 0) {
				this.InitEndAgents.add(agId);
			}
		}
		return this.InitEndAgents;
	}

	private IntArray GammaPlus(int agId) {
		IntArray higherAgents = this.bestOrder.getOrder().getElementsBefor(agId);
		IntArray neighbours = this.master.getNeighbors(agId);
		return IntArray.intersection(higherAgents, neighbours);
	}

	private boolean checkEnd() {
		this.computeInitialEndAgents();
		if (this.InitEndAgents.size() > this.currentOrders.size() || this.InitEndAgents.size() > this.acceptedMsg.size()) {
			this.master.log("Not yet receive fromall IEAgents\n"+
						"IEAgendts     : " + this.InitEndAgents + "\n"+
						"bestOrder     : " + this.bestOrder + "\n"+
						"currentOrders : " + this.currentOrders + "\n", Logger.INFO_LEVEL);
			return false;
		}
		this.master.log("\nI check the end : " + "\n"+
					"IEAgendts 	   : " + this.InitEndAgents + "\n"+
					"bestOrder 	   : " + this.bestOrder + "\n"+
					"acceptedMsg   : " + this.acceptedMsg + "\n"+
					"currentOrders : " + this.currentOrders + "\n", Logger.INFO_LEVEL);
		Instantiation solution = new Instantiation();
		for (int agId : this.InitEndAgents) {
			if (!this.currentOrders.containsKey(agId) || !this.acceptedMsg.containsKey(agId)) {
				if (Logger.LOG_DEBUG) {
					this.master.log("==>  orders or acceptedMsg do not contains " + agId, 1);
				}
				return false;
			}
			int cmp = this.bestOrder.compare(this.currentOrders.get(agId));
			if (cmp != 0 && Math.abs(cmp) <= this.bestOrder.getOrder().indexOf(agId) + 1) {
				this.master.log("==>  problem in orders \n" + agId + " => " + this.currentOrders.get(agId) + " and \n" + this.bestOrder, 1);
				return false;
			}
			Instantiation accepted = this.acceptedMsg.get(agId);
			if (solution.compatible(accepted)) {
				solution.priorityMerge(accepted);
				
			} else {
				this.master.log("non compatible accepted " + solution + " \n" + accepted, Logger.INFO_LEVEL);
				return false;
			}
		}
		for (int agId : this.InitEndAgents) {
			Instantiation accepted = this.acceptedMsg.get(agId);
			if(solution.compareTimeCounters(accepted) != 0){
				this.master.log("different counters: " + solution + " \n" + accepted, Logger.INFO_LEVEL);
				return false;
			}
		}
		this.master.log("terminate with SUCCESS : " + solution, Logger.INFO_LEVEL);
		this.master.setEndState(Commands.SUCCESS);
		this.master.broadcastSystemMessage(0);
		return true;
		// System.err.println(this.master.getEndState());
		// System.exit(0);

	}

	@Override
	public void handleSendMessage(Message msg) {}

	public void setBestOrder(CurrentOrder currentOrder) {
		if (this.bestOrder == null) {
			this.bestOrder = currentOrder;
		} else {
			if (this.bestOrder.compare(currentOrder) < 0) {
				this.bestOrder = currentOrder;
			}
		}
	}

	public String toString() {
		String term = "";
		term = "Termination detection  of :" + this.master.getModel();
		return term;
	}
}
