package simple.dcsp.agile.exp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import kernel.AbstractAgent;
import kernel.AbstractMaster;
import kernel.MasterObserver;
import kernel.com.Message;
import kernel.order.AgentComplexity;
import kernel.order.LexicographicAO;
import util.Commands;
import util.IntArray;

public class AgileMaster extends AbstractMaster {

	public static final short Topological_ID = 1;
	public static final short Topological_Deg = 2;
	public static final short Topological_MaxSucc = 3;

	public static final short All = 1;
	public static final short OrderOnly = 2;
	public static final short Neighbors = 3;
	public static final short OrderNeigh = 4;
	public static final short MinDom = 5;
	public static final short DomDeg = 6;
	public static final short DomWDeg = 7;

	public long changeTarget = 0L;
	public long keepTarget = 0L;
	public long improveTV = 0L;
	public long changeOrder = 0L;

	public long nbAssignments = 0L;
	public long nbObsoleteOk = 0L;
	public long nbObsoleteNogood = 0L;
	public long nbUpdatesFromLhs = 0L;
	public long improveTopologicalTV = 0L;

	public int nbRcvdStatistics = 0;
	// =========================== default configuration =========================== //
	private short type = OrderOnly;
	private short heuristic = MinDom;
	private short topologicalType = Topological_ID;
	private boolean topological = false;

	// ============================================================================= //

	public AgileMaster(short type) {
		this.setCentralizedAO(new LexicographicAO());
		this.addObserver(new TerminationDetector());
		this.type = type;
	}

	public AgileMaster(short type, short heuristic) {
		this(type);
		this.heuristic = heuristic;
	}

	public AgileMaster(short type, short heuristic, short topologicalType) {
		this(type, heuristic);
		this.topological = true;
		this.topologicalType = topologicalType;
	}

	@Override
	public AbstractAgent makeAgent(int agId, String name, String description) {
		return new AgileAgent(agId, name, description, this.heuristic, this.type, this.topological, this.topologicalType);
	}

	public void broadcast(Message msg, int agentId) {
		if (msg.getType().equals(Commands.AO)) {
			HashMap<Integer, AgentComplexity> ac = this.centralizedAO.getAgentsComplexity();
			for (int agId : (IntArray) msg.get(Commands.ORDER)) {
				msg.addContents(agId + "", ac.get(agId).getMaxDomain());
			}
		}
		super.broadcast(msg, agentId);
	}

	public IntArray getOrder() {
		for (MasterObserver observer : this.observers) {
			if (observer instanceof TerminationDetector) { return ((TerminationDetector) observer).order; }
		}
		return new IntArray();
	}

	/* (non-Javadoc)
	 * @see kernel.AbstractMaster#getTypesOfMessages()
	 */
	@Override
	public Collection<String> getTypesOfMessages() {
		Collection<String> ans = new ArrayList<String>();
		ans.add(Commands.OK);
		ans.add(Commands.NOGOOD);
		ans.add(Commands.ORDER);
		ans.add(Commands.TERMINATION_DETETION);
		if(type == Neighbors){
			ans.add(Commands.ADDLINK);
		}
		return ans;
	}

}
